import sys
import time

class GRS_Profile():

    def __init__(self, logfile, verbose):
        self.verbose = verbose
        if(verbose):
            self.start_time = time.localtime()
            self.logfile = logfile + "." + time.strftime("%a, %d %b %Y %H:%M:%S +0000",self.start_time).replace(" ", "_").replace(",", "").replace("+", "")

    def log_time_stamp(self, logtxt):
        if(self.verbose):
            fileHandle = open(self.logfile, 'a')
            message = time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.localtime()) + ' : ' + logtxt + '\n'
            fileHandle.write(message)
            fileHandle.close()



# This files defines a class for metadata objects of sentinel images. Large part of the work depends on python readers
# from the tops toolbox.

from doris.doris_stack.functions.burst_metadata import burst_header, burst_readfiles, burst_crop
from doris.doris_stack.main_code.resdata import ResData
import warnings
import os
import copy


class BurstMeta(ResData):
    # Class which holds and gathers information of a specific burst for sentinel 1 data.

    def __init__(self, path='', swath_no='1', pol='vv', burst_num=1, xml='', data=''):
        # Initialize variables

        # This indicates the burst number in the swath, acquisition date, centre location and the coverage of the burst (if available).
        self.burst_num = []
        self.new_burst_num = []
        self.swath_num = []
        self.burst_date = []
        self.burst_center = []
        self.burst_coverage = []
        self.burst_corners = []
        self.swath_meta = []

        # The following contain the path of xml and data file for swath and burst.
        self.swath_xml = ''
        self.swath_data = ''
        self.burst_res = ''
        self.burst_data = ''

        # orbits
        self.datapoints = []
        self.orbit_type = ''

        #############################################################

        # This function creates an swath object and searches for available data and xml files. It gives an error when
        # either the path does not exist, no data or xml files can be found or the data and xml files do not match.'

        # Create resdata for this object
        super(BurstMeta, self).__init__(type='single')

        if not xml or not data:
            xml_dir = os.path.join(path, 'annotation')
            xml = [f for f in os.listdir(xml_dir) if os.path.isfile(os.path.join(xml_dir, f))]

            data_dir = os.path.join(path, 'measurement')
            data = [f for f in os.listdir(data_dir) if os.path.isfile(os.path.join(data_dir, f))]

            # Select polarisation
            if not any(s in pol for s in ('hh','vv','hv','vh')):
                warnings.warn('Polarisation not recognized, using default (vv)')
                pol = 'vv'
            if not swath_no in ('1','2','3'):
                warnings.warn('Swath number not recognized, using default (1)')

            xml = [os.path.join(path,'annotation',x) for x in xml if x[12:14] in pol and x[6] == swath_no]
            data = [os.path.join(path,'measurement',x) for x in data if x[12:14] in pol and x[6] == swath_no]

        if type(xml) is str:
            xml = [xml]
        if type(data) is str:
            data = [data]
        if (len(xml) != 1 and type(xml) is list) or len(data) != 1:
            warnings.warn('Total number of files should be one!')
        if not os.path.exists(xml[0]) or not os.path.exists(data[0]):
            warnings.warn('Either xml or data path does not exist')
        if xml[0:-3] != data[0:-4]:
            warnings.warn('xml and data file do not correspond.')
        if not burst_num:
            warnings.warn('No burst number given')

        self.swath_xml = xml[0]
        self.swath_data = data[0]
        self.burst_num = burst_num
        self.swath_num = int(os.path.basename(xml[0])[6])

    def meta_burst(self, swath_meta=[], corners=True):
        # This function reads and stores metadata of the burst based on the swath xml file. If

        self.swath_meta = swath_meta
        readfiles = burst_readfiles(copy.deepcopy(self.swath_meta), self.burst_num, self.burst_center, self.burst_corners, self.swath_data)
        crop = burst_crop(self.swath_meta, self.burst_num, self.swath_data, self.new_burst_num)

        # Read metadata from xml and inserts in resdata of burst
        # Insert the different steps (readfiles, orbits and crop)
        self.header = burst_header('master.res')
        self.insert(readfiles, process='readfiles')
        self.insert(self.datapoints, process=self.orbit_type)
        self.insert(crop, process='crop')






import warnings
import os
import collections


class ResData(object):
    # This class hold metadata of a doris datafile and processing chain and is capable of reading from and writing to a
    # .res file used by the doris software.

    def __init__(self,filename='',type=''):
        # Initialize variables

        # Filename of resfile and type (single, interferogram)
        self.res_path = []
        self.res_type = ''

        # Processes, process_control and header of resfile
        self.processes = collections.OrderedDict()
        self.process_control = {}
        self.process_timestamp = {}
        self.process_time = {}
        self.header = {}

        #####################################################

        # Create a ResData object (single/interferogram)
        if type not in ['single','interferogram'] and not filename:
            warnings.warn('Define if results data is slave, master or interferogram')
            return
        else:
            self.res_type = type
        if filename:
            if not os.path.exists(filename):
                warnings.warn('This filename does not exist: ' + filename)
            else:
                self.res_path = filename
                self.res_read()
        else:
            if type == 'single':
                self.process_control = collections.OrderedDict([('readfiles', '0'),('leader_datapoints', '0'), ('precise_orbits', '0'), ('crop', '0'), ('sim_amplitude', '0'), ('master_timing' , '0'),
                                       ('oversample', '0'), ('resample', '0') , ('filt_azi', '0'), ('filt_range', '0'), ('NOT_USED' , '0')])
            elif type == 'interferogram':
                self.process_control = collections.OrderedDict([('coarse_orbits','0'),('coarse_correl','0'),('fine_coreg','0'),('timing_error','0'),('dem_assist','0'),
                                   ('comp_coregpm','0'),('interfero','0'),('coherence','0'),('comp_refphase','0'),('subtr_refphase','0'),
                                   ('comp_refdem','0'),('subtr_refdem','0'),('filtphase','0'),('unwrap','0'),('est_orbits','0'),('slant2h','0'),
                                   ('geocoding','0'),('dinsar','0'),('NOT_USED2','0')])

    def res_read(self):
        self.meta_reader()
        self.process_reader()

    def meta_reader(self):
        # This function
        with open(self.res_path) as resfile:
            splitter = ':'
            temp = collections.OrderedDict()
            row = 0
            for line in resfile:
                try:
                    ## Filter out rubbish
                    if line == '\n':
                        continue
                    elif 'Start_process_control' in line:
                        self.header = temp
                        temp = collections.OrderedDict()
                    elif 'End_process_control' in line:
                        self.process_control = temp
                        break
                    elif splitter in line and line[0] is not '|' and line[0] is not '\t' :
                        # Split line if possible and add to dictionary
                        l_split = line.split(splitter)
                        temp[l_split[0].strip()] = l_split[1].strip()
                    else:
                        name = 'row_' + str(row)
                        row += 1
                        temp[name] = [line]

                except:
                    print 'Error occurred at line: ' + line

    def process_reader(self,processes = ''):
        # This function reads random processes based on standard buildup of processes in res files.
        # leader_datapoints can be one of the processes, although it will not appear in the process_control in a .res file
        # If loc is true, it will only return the locations where different processes start.

        if not processes:
            processes = self.process_control.keys()

        processes.append('leader_datapoints')
        process = ''

        with open(self.res_path) as resfile:
            # Start at row zero and with empty list
            temp = collections.OrderedDict()
            row = 0
            line_no = -1
            timestamp = False
            timestamp_line = 0
            for line in resfile:
                try:
                    line_no += 1
                    # Filter out rubbish
                    if '|'in line[0]:
                        continue
                    elif '**' in line:
                        continue
                    elif line == '\n':
                        continue

                    # Check if timestamp
                    if ' *===========' in line:
                        # First line of time stamp
                        temp = collections.OrderedDict()
                        timestamp = True
                        row = 0
                        continue
                    elif ' *-----------' in line:
                        timestamp = False
                        timestamp_data = temp
                        timestamp_line = line_no + 5
                        continue

                    # Check if process
                    if '*' in line[0]:
                        if line.replace('*_Start_', '').split(':')[0].strip() in processes:
                            process = line.replace('*_Start_', '').split(':')[0].strip()
                            temp = collections.OrderedDict()
                            row = 0; space = [0]; space_r = [0,0,0,0,0,0,0,0]

                            # Finally save the timestamp if it exists
                            if line_no == timestamp_line:
                                self.process_timestamp[process] = timestamp_data
                            else:
                                self.process_timestamp[process] = ''

                        elif line.replace('* End_', '').split(':')[0] == process:
                            self.processes[process] = temp
                            temp = collections.OrderedDict()
                            process = ''
                        continue

                    # Save line
                    if timestamp is True:
                        # Save rows in timestamp
                        row_name = 'row_' + str(row)
                        temp[row_name] = line
                        if row == 1:
                            self.process_time[process] = line.split(':', 1)[1].strip()
                        row += 1
                    elif process:
                        # If we are in a process output line
                        # Split line using ':' , '=' or spaces (tables)
                        # variable space and space row define the future spacing in every processing step in a res file.

                        if process == 'coarse_orbits':
                            # Add some code for a strange exception in coarse_orbits
                            if '//' in line:
                                temp[line.split()[0]] = line.split()[1:]
                            else:
                                l_split = line.replace('=',':').split(':')
                                temp[l_split[0].strip()] = l_split[1].strip()

                        elif ':' in line:
                            l_split = line.split(':',1)
                            temp[l_split[0].strip()] = l_split[1].strip()
                        else:
                            # If the line does not contain a : it is likely a table.
                            l_split = line.replace('\t',' ').split()
                            row_name = 'row_' + str(row)
                            temp[row_name] = [l_split[i].strip() for i in range(len(l_split))]
                            row += 1

                except:
                    print 'Error occurred at line: ' + line

    def process_spacing(self,process=''):

        spacing = 0
        table_spacing = [0,0,0,0,0,0,0]

        dat = self.processes[process]

        for key in dat.keys():
            spacing = max(len(key) + 8, spacing)

            if key.startswith('row'):
                n=0
                for val in self.processes[process][key]:
                    table_spacing[n] = max(len(val) + 3, table_spacing[n])
                    n += 1
        spacing = [spacing]

        return spacing, table_spacing

    def del_process(self,process=''):
        # function deletes one or multiple processes from the corresponding res file

        if isinstance(process, basestring): # one process
            if not process in self.process_control.keys():
                warnings.warn('The requested process does not exist! (or processes are not read jet, use self.process_reader): ' + str(process))
                return
        elif isinstance(process, list): # If we use a list
            for proc in process:
                if not proc in self.process_control.keys():
                    warnings.warn('The requested process does not exist! (or processes are not read jet, use self.process_reader): ' + str(proc))
                    return
        else:
            warnings.warn('process should contain either a string of one process or a list of multiple processes: ' + str(process))

        # Now remove the process and write the file again.
        if isinstance(process, basestring): # Only one process should be removed
            self.process_control[process] = '0'
            del self.processes[process]
        else:
            for proc in process:
                self.process_control[proc] = '0'
                del self.processes[proc]

    def write(self,new_filename=''):
        # Here all the available information acquired is written to a new resfile. Generally if information is manually
        # added or removed and the file should be created or created again. (For example the readfiles for Sentinel 1
        # which are not added yet..)

        if not new_filename and not self.res_path:
            warnings.warn('Please specify filename: ' + str(new_filename))
            return
        elif not new_filename:
            new_filename = self.res_path
        if not self.process_control or not self.processes:
            warnings.warn('Every result file needs at least a process control and one process to make any sense: ' + str(new_filename))

        # Open file and write header, process control and processes
        self.res_path = new_filename
        f = open(new_filename,"w")

        # Write the header:
        if self.header:
            spacing = [40]
            for key in self.header.keys():
                if 'row' in key:       # If it is just a string
                    f.write(self.header[key][0])
                else:                   # If the key should included
                    f.write((key + ':').ljust(spacing[0]) + self.header[key] + '\n')

        # Write the process control
        for i in range(3):
            f.write('\n')
        f.write('Start_process_control\n')
        for process in self.process_control.keys():
            if process != 'leader_datapoints':  # leader_datapoints is left out in process control
                f.write((process + ':\t\t') + str(self.process_control[process]) + '\n')
        f.write('End_process_control\n')

        # Then loop through all the processes
        for process in [p for p in self.processes.keys()]:
            # First check for a timestamp and add it if needed.
            if self.process_timestamp[process]:
                for i in range(2):
                    f.write('\n')
                f.write('   *====================================================================* \n')
                for key in self.process_timestamp[process].keys():
                    f.write(self.process_timestamp[process][key])
                f.write('   *--------------------------------------------------------------------* \n')

            # Then write the process itself
            if process == 'coarse_orbits':
                spacing = [45]
                spacing_row = [15,10,15]
            else:
                spacing, spacing_row = self.process_spacing(process)
            data = self.processes[process]

            for i in range(3):
                f.write('\n')
            f.write('******************************************************************* \n')
            f.write('*_Start_' + process + ':\n')
            f.write('******************************************************************* \n')

            for line_key in self.processes[process].keys():
                if 'row' in line_key:  # If it is a table of consists of several different parts
                    line = ''.join([(' ' + data[line_key][i]).replace(' -','-').ljust(spacing_row[i]) for i in range(len(data[line_key]))])
                    f.write(line + '\n')
                elif process == 'coarse_orbits':  # the coarse orbits output is different from the others.
                    if 'Control point' in line_key: # Special case coarse orbits...
                        f.write((line_key + ' =').ljust(spacing[0]) + str(self.processes[process][line_key]) + '\n')
                    elif not isinstance(data[line_key], basestring): # Another special case
                        f.write(line_key.ljust(spacing_row[0]) + (data[line_key][0]).ljust(spacing_row[1]) +
                                data[line_key][1].ljust(spacing_row[2]) + ' '.join(data[line_key][2:]) + '\n')
                    elif isinstance(data[line_key], basestring): # Handle as in normal cases
                        f.write((line_key + ':').ljust(spacing[0]) + str(self.processes[process][line_key]) + '\n')
                else: # If it consists out of two parts
                    f.write((line_key + ':').ljust(spacing[0]) + str(self.processes[process][line_key]) + '\n')

            f.write('******************************************************************* \n')
            f.write('* End_' + process + ':_NORMAL\n')
            f.write('******************************************************************* \n')
        f.close()

        # Read the locations in the new file
        self.process_reader()

    def insert(self,data,process,variable=''):
        # This function inserts a variable or a process which does not exist at the moment
        processes = self.process_control.keys()
        processes.extend(['header','leader_datapoints'])

        if process not in processes:
            warnings.warn('This process does not exist for this datatype: ' + str(process))
            return

        # If a full process is added
        if not variable:
            if self.process_control[process] == '1':
                warnings.warn('This process already exists! Use the update function: ' + str(process))
                return
            elif self.process_control[process] == '0':
                self.process_control[process] = '1'
                self.processes[process] = data
                self.process_timestamp[process] = ''

        # A variable is added
        if variable:
            if variable in self.processes[process].keys():
                warnings.warn('This variable already exists! Use the update function: ' + str(variable))
                return
            elif not self.processes[process][variable]:
                self.processes[process][variable] = data

    def delete(self,process,variable=''):
        # This function deletes a variable or a process which does exist at the moment
        processes = self.process_control.keys()
        processes.extend(['header','leader_datapoints'])

        if process not in processes:
            warnings.warn('This process does not exist for this datatype: ' + str(process))
            return

        # If a full process is deleted
        if not variable:
            if self.process_control[process] == '0':
                warnings.warn('This process does not exist: ' + str(process))
                return
            elif self.process_control[process] == '1':
                self.process_control[process] = '0'
                del self.processes[process]
                del self.process_timestamp[process]

        # A variable is deleted
        if variable:
            if not variable in self.processes[process].keys():
                warnings.warn('This variable does not exist: ' + str(variable))
                return
            else:
                del self.processes[process][variable]

    def update(self,data,process,variable=''):
        # This function updates a variable or a process which does exist at the moment
        processes = self.process_control.keys()
        processes.extend(['header','leader_datapoints'])

        if not process in processes:
            warnings.warn('This process does not exist for this datatype: ' + str(process))
            return

        # If a full process is added
        if not variable:
            if self.process_control[process] == '1':
                self.processes[process] = data
            elif self.process_control[process] == '0':
                warnings.warn('This process does not exist. Use the insert function: ' + str(process))
                return
        # A variable is added
        if variable:
            if variable in self.processes[process].keys():
                self.processes[process][variable] = data
            elif not self.processes[process][variable]:
                warnings.warn('This variable does not exist. Use the insert function: ' + str(variable))
                return

    def request(self,process,variable=''):
        # This function updates a variable or a process which does exist at the moment
        processes = self.process_control.keys()
        processes.extend(['header','leader_datapoints'])

        if not process in processes:
            warnings.warn('This process does not exist for this datatype: ' + str(process))
            return

        # If a full process is added
        if not variable:
            if self.process_control[process] == '1':
                data = self.processes[process]
            elif self.process_control[process] == '0':
                warnings.warn('This process does not exist: ' + str(process))
                return
        # A variable is added
        if variable:
            if variable in self.processes[process].keys():
                data = self.processes[process][variable]
            elif not self.processes[process][variable]:
                warnings.warn('This variable does not exist: ' + str(variable))
                return

        return data


import os
import sys


class DorisParameters_Path(object):

    def set(self, doris_parameters_path):
            if(os.path.exists(doris_parameters_path)):
                sys.path.append(os.path.split(doris_parameters_path)[0])
                print 'dorisparameter path: ' + doris_parameters_path
            else:
                print 'dorisparameter path: ' + doris_parameters_path + ' not a valid path'



import xml.etree.ElementTree as ET
import os

def init_cfg():
    template_xml_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doris_config_template.xml')
    xml_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doris_config.xml')
    tree = ET.parse(template_xml_file)
    settings = tree.getroot()

    settings.find('.source_path').text = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

    input = False
    while input == False:
        user_input = raw_input("Enter the path to doris: ")
        if os.path.exists(user_input) and user_input.endswith('doris'):
            settings.find('.doris_path').text = user_input
            input = True
        else:
            print('The path is incorrect, use another path')

    input = False
    while input == False:
        user_input = raw_input("Enter the path to cpxfiddle: ")
        if os.path.exists(user_input) and user_input.endswith('cpxfiddle'):
            settings.find('.cpxfiddle_path').text = user_input
            input = True
        else:
            print('The path is incorrect, use another path')

    input = False
    while input == False:
        user_input = raw_input("Enter the path to snaphu: ")
        if os.path.exists(user_input) and user_input.endswith('snaphu'):
            settings.find('.snaphu_path').text = user_input
            input = True
        else:
            print('The path is incorrect, use another path')

    # Now create the password file.
    user_input = raw_input("Enter your username for scihub (https://scihub.copernicus.eu/dhus/#/self-registration)")
    if len(user_input) > 0:
        settings.find('.scihub_username').text = user_input
    else:
        print('Username field is empty, you can change it later in the doris_config.xml file')

    user_input = raw_input("Enter your password for scihub ")
    if len(user_input) > 0:
        settings.find('.scihub_password').text = user_input
    else:
        print('Password field is empty, you can change it later in the doris_config.xml file')

    user_input = raw_input("Enter your username for srtm download (https://urs.earthdata.nasa.gov/users/new/)")
    if len(user_input) > 0:
        settings.find('.usgs_username').text = user_input
    else:
        print('Username field is empty, you can change it later in the doris_config.xml file')

    user_input = raw_input("Enter your password for srtm download ")
    if len(user_input) > 0:
        settings.find('.usgs_password').text = user_input
    else:
        print('Password field is empty, you can change it later in the doris_config.xml file')

    print('Doris is initialized. If you want to make changes later, you can change the doris_config.xml file' +
          ' or run this script again')

    tree.write(open(xml_file, 'w'))

# Actually execute the code...
if __name__ == "__main__":

    # Initialize...
    init_cfg()



import os
import numpy as np
import gdal
from gdalconst import *
import sys

def get_ramp(res_file, resampled=0, type='chirp'):
    # Read information
    ################################################################################

    #FM
    t0_FM = np.float64(get_parameter('FM_reference_range_time', res_file,1))
    c0_FM = np.float64(get_parameter('FM_polynomial_constant_coeff (Hz, early edge)', res_file,1))
    c1_FM = np.float64(get_parameter('FM_polynomial_linear_coeff (Hz/s, early edge)', res_file,1))
    c2_FM = np.float64(get_parameter('FM_polynomial_quadratic_coeff (Hz/s/s, early edge)', res_file,1))

    #DC
    azimuthTime_DC = get_parameter('DC_reference_azimuth_time', res_file,3)
    azimuthTime_DC = np.float64(azimuthTime_DC[0])*3600+float(azimuthTime_DC[1])*60+float(azimuthTime_DC[2])

    t0_DC = np.float64(get_parameter('DC_reference_range_time',res_file,1))
    c0_DC = np.float64(get_parameter('Xtrack_f_DC_constant (Hz, early edge)', res_file,1))
    c1_DC = np.float64(get_parameter('Xtrack_f_DC_linear (Hz/s, early edge)', res_file,1))
    c2_DC = np.float64(get_parameter('Xtrack_f_DC_quadratic (Hz/s/s, early edge)', res_file,1))

    Ks = np.float64(get_parameter('Azimuth_steering_rate (deg/s)', res_file,1))

    # Image sampling parameters
    Taz_start = get_parameter('First_pixel_azimuth_time (UTC)', res_file,3)
    Taz_start = np.float64(Taz_start[0])*3600+float(Taz_start[1])*60+float(Taz_start[2])

    Trg_start = np.float64(get_parameter('Range_time_to_first_pixel (2way) (ms)', res_file,1))*1e-3
    fsRg = np.float64(get_parameter('Range_sampling_rate (computed, MHz)', res_file,1))

    dt_az = np.float64(get_parameter('Azimuth_time_interval (s)', res_file,1))
    dt_rg = 1/fsRg/1e6

    # Number of lines
    lNum = int(get_parameter('Number_of_lines_original', res_file,1))

    if resampled == 1:
        l0 = int(get_parameter('First_line (w.r.t. original_master)', res_file,2,'*_Start_resample','* End_resample:_NORMAL'))
        lN = int(get_parameter('Last_line (w.r.t. original_master)', res_file,2,'*_Start_resample','* End_resample:_NORMAL'))
        p0 = int(get_parameter('First_pixel (w.r.t. original_master)', res_file,2,'*_Start_resample','* End_resample:_NORMAL'))
        pN = int(get_parameter('Last_pixel (w.r.t. original_master)', res_file,2,'*_Start_resample','* End_resample:_NORMAL'))
    else:
        l0 = int(get_parameter('First_line (w.r.t. original_image)', res_file, 1))
        lN = int(get_parameter('Last_line (w.r.t. original_image)', res_file, 1))
        p0 = int(get_parameter('First_pixel (w.r.t. original_image)', res_file, 1))
        pN = int(get_parameter('Last_pixel (w.r.t. original_image)', res_file, 1))

    # Get resampled Slv size
    Naz_res = lN-l0+1
    Nrg_res = pN-p0+1

    if resampled == 1:
        # Read the resampled image and slave coordinates in master geometry
        #################################################################################

        Path_MFF_HDR   ='rsmp_orig_slave_pixel'+'.hdr'
        Link_DATA      ='rsmp_orig_slave_pixel'+'.r00'  # the default format should be r00
        Link_rsmp_orig_slave_pixel ='rsmp_orig_slave_pixel.raw'

        if (os.path.isfile(Path_MFF_HDR)):
            os.remove(Path_MFF_HDR)
        if (os.path.isfile(Link_DATA)):
            os.remove(Link_DATA)

        RAW_DATA_ABSOLUTE_PATH=os.path.abspath(Link_rsmp_orig_slave_pixel)
        print "RAW_DATA_ABSOLUTE_PATH=", RAW_DATA_ABSOLUTE_PATH
        os.symlink(RAW_DATA_ABSOLUTE_PATH,Link_DATA)

        outStream      = open(Path_MFF_HDR,'w')
        outStream.write('IMAGE_FILE_FORMAT = MFF\n')
        outStream.write('FILE_TYPE = IMAGE\n')
        outStream.write('IMAGE_LINES = %d\n' % int(Naz_res))
        outStream.write('LINE_SAMPLES = %d\n'% int(Nrg_res))
        outStream.write('BYTE_ORDER = LSB\n')
        outStream.write('END\n')
        outStream.close()

        PixRgGrid = freadbk(Path_MFF_HDR,1, 1,int(Naz_res),int(Nrg_res))
        PixRgGrid = PixRgGrid.astype(np.float64)

        if (os.path.isfile(Path_MFF_HDR)):
            os.remove(Path_MFF_HDR)
        if (os.path.isfile(Link_DATA)):
            os.remove(Link_DATA)
        #################################################################################

        Path_MFF_HDR   ='rsmp_orig_slave_line'+'.hdr'
        Link_DATA      ='rsmp_orig_slave_line'+'.r00'
        Link_rsmp_orig_slave_line ='rsmp_orig_slave_line.raw'

        if (os.path.isfile(Path_MFF_HDR)):
            os.remove(Path_MFF_HDR)
        if (os.path.isfile(Link_DATA)):
            os.remove(Link_DATA)


        RAW_DATA_ABSOLUTE_PATH=os.path.abspath(Link_rsmp_orig_slave_line)
        print "RAW_DATA_ABSOLUTE_PATH=", RAW_DATA_ABSOLUTE_PATH
        os.symlink(RAW_DATA_ABSOLUTE_PATH,Link_DATA)

        outStream      = open(Path_MFF_HDR,'w')
        outStream.write('IMAGE_FILE_FORMAT = MFF\n')
        outStream.write('FILE_TYPE = IMAGE\n')
        outStream.write('IMAGE_LINES = %d\n' % int(Naz_res))
        outStream.write('LINE_SAMPLES = %d\n'% int(Nrg_res))
        outStream.write('BYTE_ORDER = LSB\n')
        outStream.write('END\n')
        outStream.close()

        PixAzGrid = freadbk(Path_MFF_HDR,1, 1,int(Naz_res),int(Nrg_res))
        PixAzGrid=PixAzGrid.astype(np.float64)

        if (os.path.isfile(Path_MFF_HDR)):
            os.remove(Path_MFF_HDR)
        if (os.path.isfile(Link_DATA)):
            os.remove(Link_DATA)

        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        #% Prepare azimuth and range grids %
        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        TrgGrid = Trg_start + (PixRgGrid-1) * dt_rg
        TazGrid = (PixAzGrid-1) * dt_az - (lNum/2 * dt_az)

        del PixAzGrid, PixRgGrid

    elif resampled == 0:
        Tvect_rg = Trg_start + np.arange(p0-1,pN) * dt_rg
        Tvect_az = np.arange(l0-1,lN) * dt_az - (lNum/2 * dt_az)
        Tvect_az = Tvect_az[:, None]

        TrgGrid = np.tile(Tvect_rg, (Naz_res, 1))
        TazGrid = np.tile(Tvect_az, (1, Nrg_res))
        
    else:
        print 'variable resampled can only be 0 or 1!'
        return

    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #% From S-1 steering rate and orbit information %
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%%%%%%%%%%%%%%%%%%%%%
    #% Orbit information %
    #%%%%%%%%%%%%%%%%%%%%%

    orbit_number     = int(get_parameter('NUMBER_OF_DATAPOINTS',res_file,1))
    orbit_time       = np.zeros(orbit_number,dtype=np.float64)
    orbit_velocity_x = np.zeros(orbit_number,dtype=np.float64)
    orbit_velocity_y = np.zeros(orbit_number,dtype=np.float64)
    orbit_velocity_z = np.zeros(orbit_number,dtype=np.float64)
    orbit_info       = get_parameter('NUMBER_OF_DATAPOINTS',res_file,4)

    orbit_info=orbit_info.split('\n')

    for row in range(orbit_number):
        orbit_time_position=orbit_info[row]
        orbit_time[row]=float(orbit_time_position.strip().split()[0])
        orbit_velocity_x[row]=float(orbit_time_position.strip().split()[1])
        orbit_velocity_y[row]=float(orbit_time_position.strip().split()[2])
        orbit_velocity_z[row]=float(orbit_time_position.strip().split()[3])
    orbit_velocity = np.sqrt(np.diff(orbit_velocity_x)**2+np.diff(orbit_velocity_y)**2+np.diff(orbit_velocity_z)**2)/np.diff(orbit_time)

    # Compute Nominal DC for the whole burst
    # Compute FM rate along range
    Kfm = c0_FM + c1_FM*(TrgGrid-t0_FM) + c2_FM*(TrgGrid-t0_FM)**2
    Kfm_0 = c0_FM + c1_FM*(Trg_start-t0_FM) + c2_FM*(Trg_start-t0_FM)**2

    # Compute DC along range at reference azimuth time (azimuthTime)
    Df_AzCtr = c0_DC + c1_DC*(TrgGrid-t0_DC) + c2_DC*(TrgGrid-t0_DC)**2
    f_DC_ref_0 = c0_DC + c1_DC*(Trg_start-t0_DC) + c2_DC*(Trg_start-t0_DC)**2
    del TrgGrid

    # From S-1 steering rate and orbit information %
    # Computes sensor velocity from orbits
    C_lambda=np.float64(get_parameter('Radar_wavelength (m)',res_file,1))

    # Frequency rate
    Ks_hz = 2*np.mean(orbit_velocity)/C_lambda*Ks/180*np.pi

    # Time ratio
    alpha_nom = 1 - Ks_hz/Kfm

    # DC Azimuth rate [Hz/s]
    DR_est = Ks_hz/alpha_nom
    del Ks_hz, alpha_nom

    # Reference time
    az_DC = -(Df_AzCtr / Kfm) + (f_DC_ref_0 / Kfm_0)
    del Kfm, Kfm_0
    Taz_vec = TazGrid - az_DC
    del az_DC

    #% Generate inverse chirp %
    if type == 'chirp':
        data = np.exp(1j*2*np.pi*(DR_est/2*Taz_vec+Df_AzCtr)*Taz_vec)
    elif type == 'DC':
        data = Df_AzCtr + Taz_vec * DR_est 
    else:
        print 'Choose either chirp or DC for type'
        return

    return data


def get_parameter(First_param,file_name,format_flag=1,Second_param=None,Third_param=None):
    Read_contine_flag=0
    orbit_info=""
    value=None
    for line in open(file_name):
        if format_flag==1:
            if not (line.find(First_param)):
                index=line.find(':')
                value=(line[(index+1):].strip(' \n\t'))
                return value

        if format_flag==2:
            if not(line.find(Second_param)):
                Read_contine_flag=1
            if (Read_contine_flag==1) and (not (line.find(First_param))):  #Be careful
                index=line.find(':')
                value=(line[(index+1):].strip(' \n\t'))
                continue
            if Read_contine_flag==1  and (not(line.find(Third_param))):  #Be careful
                Read_contine_flag=0
                return value


        if format_flag==3:
            if not (line.find(First_param)):
                index=line.find(':')
                pixel_time=(line[(index+1):].strip(' \n\t')).split(' ')[1].split(':')
                return pixel_time


        if format_flag==4:

            if not (line.find(First_param)):
                index=line.find(':')
                value=int(line[(index+1):].strip(' \n\t'))
                Read_contine_flag=1
                continue
            if (Read_contine_flag>=1):
                orbit_info=orbit_info+line
                Read_contine_flag=Read_contine_flag+1
                if (Read_contine_flag==(value+1)):
                    return orbit_info
################################################################################


###############################################################################

def freadbk(path_file,line_start=1, pixels_start=1,nofLines1=None,nofPixels1=None):
    #Driver
    driver=gdal.GetDriverByName('MFF')
    driver.Register()
    gdal.AllRegister()
    thisBurstData_file=gdal.Open(path_file,GA_ReadOnly)
    if thisBurstData_file is None:
        print 'Could not open'+Path_MFF_HDR
        sys.exit(1)
    #print 'Driver: ', thisBurstData_file.GetDriver().ShortName,'/', \
    #      thisBurstData_file.GetDriver().LongName
    #print 'Size is ',thisBurstData_file.RasterXSize,'x',thisBurstData_file.RasterYSize, \
    #      'x',thisBurstData_file.RasterCount
    #print 'Projection is ',thisBurstData_file.GetProjection()
    geotransform = thisBurstData_file.GetGeoTransform()
    if not geotransform is None:
        print 'Origin = (',geotransform[0], ',',geotransform[3],')'
        print 'Pixel Size = (',geotransform[1], ',',geotransform[5],')'

    cint_srd=thisBurstData_file.GetRasterBand(1)
    #print 'Band Type=',gdal.GetDataTypeName(cint_srd.DataType)

    if cint_srd.GetOverviewCount() > 0:
            print 'Band has ', cint_srd.GetOverviewCount(), ' overviews.'
    thisBurstData= cint_srd.ReadAsArray(int(pixels_start-1),int(line_start-1),nofPixels1,nofLines1)
    return thisBurstData
##################################################################################



# This script is used to remove the residual ramp in the ifgs of individual bursts based on ESD estimates.

import numpy as np
import os, sys

if __name__ == "__main__":
    # If calling script directly we have to load the package first to our python path
    folder = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
    print(folder)
    sys.path.extend([folder])

from resdata import ResData


def remove_ramp(file, angle_pixel):
    # Remove ramp from burst

    res_file = 'master.res'
    res_dat = ResData(res_file, 'master')
    crop = res_dat.processes['crop']
    lines = int(crop['Last_line (w.r.t. original_image)']) - int(crop['First_line (w.r.t. original_image)']) + 1
    pixels = int(crop['Last_pixel (w.r.t. original_image)']) - int(crop['First_pixel (w.r.t. original_image)']) + 1

    n = np.arange(lines)
    phase_diff = n * angle_pixel
    complex_diff = np.cos(phase_diff).astype('complex64') + 1j * np.sin(phase_diff).astype('complex64')

    dat_file = np.memmap(file, dtype='complex64', mode='r+', shape=(lines, pixels))
    p_before = np.nanmean(np.angle(dat_file))
    print('Average phase before is ' + str(p_before))

    dat_file[:, :] = dat_file * complex_diff.conj()[:, None]
    p_after = np.nanmean(np.angle(dat_file))
    print('Average phase after is ' + str(p_after))
    dat_file.flush()

if __name__ == "__main__":
    # If calling script directly we run the remove_ramp function.

    if len(sys.argv) == 3:
        file            = sys.argv[1]
        angle_pixel     = sys.argv[2]  # Use 1 if needed, use 0 if not
    else:
        sys.exit('usage: burst_folder, file, angle_per_pixel')

    print('file we will deramp is ' + file)
    print('the angle per pixel is ' + angle_pixel)

    angle_pixel = float(angle_pixel)
    remove_ramp(file, angle_pixel)




# Based on the orbit of the swath the orbits of the individual burst is calculated.

from orbit_coordinates import lph2xyz, xyz2ell, intrp_orbit
import os
import numpy as np
import collections
from datetime import datetime
from doris.doris_stack.functions.resdata import ResData
from shapely.geometry import Polygon


def burst_header(resID):

    meta = collections.OrderedDict()

    meta['row_1'] = ['===============================================\n']
    meta['MASTER RESULTFILE:'] = resID
    meta['Created by'] = 'G.Mulder TU Delft'
    meta['row_2'] = 'Doris (Delft o-o Radar Interferometric Software)'
    meta['Version'] = 'Version (2015) (For TOPSAR)'
    meta['FFTW library'] = 'used'
    meta['VECLIB library'] = 'not used'
    meta['LAPACK library'] = 'not used'
    meta['Compiled at'] = 'XXXXXXXX'
    meta['By GUN gcc'] = 'XXXXXXXX'
    meta['row_3'] = ['===============================================\n']

    return meta


def burst_readfiles(meta, burst_num, burst_center, burst_border, swath_data):
    # First copy swath metadata for burst and create a georef dict which stores information about the geo reference of
    # the burst.
    meta['Burst_number_index'] = str(burst_num)
    aux = meta['aux']
    aux['azimuthPRF'] = [meta['Pulse_Repetition_Frequency (computed, Hz)']]
    meta.pop('aux')

    # First find coordinates of center and optionally the corners
    meta['Scene_centre_longitude'] = str(burst_center[0])
    meta['Scene_centre_latitude'] = str(burst_center[1])
    meta['Scene_ul_corner_latitude'] = str(burst_border[0][1])
    meta['Scene_ur_corner_latitude'] = str(burst_border[1][1])
    meta['Scene_lr_corner_latitude'] = str(burst_border[2][1])
    meta['Scene_ll_corner_latitude'] = str(burst_border[3][1])
    meta['Scene_ul_corner_longitude'] = str(burst_border[0][0])
    meta['Scene_ur_corner_longitude'] = str(burst_border[1][0])
    meta['Scene_lr_corner_longitude'] = str(burst_border[2][0])
    meta['Scene_ll_corner_longitude'] = str(burst_border[3][0])

    # Find doppler centroid frequency and azimuth reference time
    doppler_times = [np.datetime64(aux['doppler_azimuth_Time'][i] + '-00') for i in range(len(aux['doppler_azimuth_Time']))]
    frequency_times = [np.datetime64(aux['doppler_azimuth_Time'][i] + '-00') for i in range(len(aux['doppler_azimuth_Time']))]
    burst_start_time = np.datetime64(aux['azimuthTimeStart'][burst_num-1] + '-00')
    meta['First_pixel_azimuth_time (UTC)'] = burst_start_time.astype(datetime).strftime('%Y-%b-%d %H:%M:%S.%f')

    # First index after start burst for doppler and azimuth
    doppler_id = np.where(doppler_times > burst_start_time)
    frequency_id = np.where(frequency_times > burst_start_time)

    # Assign DC values to metadata
    if len(doppler_id[0]) > 0:
        doppler_id = np.min(doppler_id)
        parameter = aux['dopplerCoeff'][doppler_id].split()
        meta['DC_reference_azimuth_time'] = np.datetime64(aux['doppler_azimuth_Time'][doppler_id] + '-00').astype(datetime).strftime('%Y-%b-%d %H:%M:%S.%f')
        meta['DC_reference_range_time'] = aux['doppler_range_Time'][doppler_id]
    else:
        parameter = ['0.0','0.0','0.0']
        meta['DC_reference_azimuth_time'] = '0.0 0.0'
        meta['DC_reference_range_time'] = aux['doppler_range_Time'][-1]
    # Assign parameters
    meta['Xtrack_f_DC_constant (Hz, early edge)'] = parameter[0]
    meta['Xtrack_f_DC_linear (Hz/s, early edge)'] = parameter[1]
    meta['Xtrack_f_DC_quadratic (Hz/s/s, early edge)'] = parameter[2]

    # Assign FM values to metadata
    if len(frequency_id[0]) > 0:
        frequency_id = np.min(frequency_id)
        if aux['azimuthFmRate_c0']:
            parameter = [aux['azimuthFmRate_c0'][frequency_id],aux['azimuthFmRate_c1'][frequency_id],aux['azimuthFmRate_c2'][frequency_id]]
        else:
            parameter = aux['azimuthFmRatePolynomial'][frequency_id].split()
        meta['FM_reference_azimuth_time'] = np.datetime64(aux['azimuthFmRate_reference_Azimuth_time'][frequency_id] + '-00').astype(datetime).strftime('%Y-%b-%d %H:%M:%S.%f')
        meta['FM_reference_range_time'] = aux['azimuthFmRate_reference_Range_time'][frequency_id]
    else:
        parameter = ['0.0','0.0','0.0']
        meta['doppler_azimuth_Time'] = '0.0 0.0'
        meta['FM_reference_range_time'] = aux['azimuthFmRate_reference_Range_time'][-1]
    # Assign parameters
    meta['FM_polynomial_constant_coeff (Hz, early edge)'] = parameter[0]
    meta['FM_polynomial_linear_coeff (Hz/s, early edge)'] = parameter[1]
    meta['FM_polynomial_quadratic_coeff (Hz/s/s, early edge)'] = parameter[2]

    # Add information about swath
    meta['row_1'] = ['******************************************************************']
    meta['Datafile'] = os.path.basename(swath_data)
    meta['Dataformat'] = 'tiff'
    meta['Number_of_lines_original'] = aux['imageLines'][0]
    meta['Number_of_pixels_original'] = aux['imagePixels'][0]
    meta['deramp'] = '0'
    meta['reramp'] = '0'
    meta['ESD_correct'] = '0'

    return meta


def burst_crop(meta,burst_num,swath_data,new_burst_num):
    # This function returns a description of the crop files part, which defines how the burst is cropped out of the
    # swath data. This function is generally called when the .res and raw data is written to the datastack and uses one
    # of the outputs from the burst_readfiles

    crop = collections.OrderedDict()

    last_sample =  [int(x) for x in meta['aux']['lastValidSample'][burst_num-1].split()]
    first_sample = [int(x) for x in meta['aux']['firstValidSample'][burst_num-1].split()]

    swath_data = os.path.basename(swath_data)
    crop['Data_output_file'] = 'slave_iw_' + swath_data[6] + '_burst_' + str(new_burst_num) + '.raw'
    crop['Data_output_format'] = 'complex_short'

    # Start line of this burst in total swath product
    lines = meta['aux']['imageLines'][0]
    extra_lines = int(lines) * (burst_num-1)

    # All coordinates are one based (e.g. we start at pixel 1)
    halfway = int(len(last_sample)/2)

    crop['First_line (w.r.t. tiff_image)'] = str(1 + last_sample[:halfway].count(-1) + extra_lines)
    crop['Last_line (w.r.t. tiff_image)'] = str(len(last_sample) - last_sample[halfway:].count(-1) + extra_lines)
    crop['First_line (w.r.t. original_image)'] = str(1 + last_sample[:halfway].count(-1))
    crop['Last_line (w.r.t. original_image)'] = str(len(last_sample) - last_sample[halfway:].count(-1))
    crop['First_pixel (w.r.t. original_image)'] = str(max(first_sample))
    crop['Last_pixel (w.r.t. original_image)'] = str(max(last_sample))

    return crop


def center_shape_from_res(resfile):
    # This function reads the shape and center of a burst from a .res file.

    res = ResData(resfile)
    res.res_read()
    meta = res.processes['readfiles']

    center = (float(meta['Scene_centre_longitude']), float(meta['Scene_centre_latitude']))
    ul = (float(meta['Scene_ul_corner_longitude']), float(meta['Scene_ul_corner_latitude']))
    ur = (float(meta['Scene_ur_corner_longitude']), float(meta['Scene_ur_corner_latitude']))
    lr = (float(meta['Scene_lr_corner_longitude']), float(meta['Scene_lr_corner_latitude']))
    ll = (float(meta['Scene_ll_corner_longitude']), float(meta['Scene_ll_corner_latitude']))

    coverage = Polygon([ul, ur, lr, ll])

    return center, coverage


'''
	GrsConfig defines paths that are local to the source tree.
	They are copied into DorisParameters for use in Doris python scripts
'''

import xml.etree.ElementTree as ET
import sys, os

class DorisConfig(object):

    def __init__(self):

        xml_file = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))), 'install/doris_config.xml')
        tree = ET.parse(xml_file)
        settings = tree.getroot()

        self.source_path = settings.find('.source_path').text
        self.doris_path = settings.find('.doris_path').text
        self.cpxfiddle_path = settings.find('.cpxfiddle_path').text

        self.job_handler_script = self.source_path + "/doris_stack/main_code/jobHandlerScript"
        self.function_path = self.source_path + "/doris_stack/functions/"
        self.main_code_path = self.source_path + "/doris_stack/main_code/"

        # Extend path
        sys.path.extend([self.function_path])
        sys.path.extend([self.main_code_path])



# This script can be used to cleanup a datastack
# In this script we use the following rational
# 1. Data from the master date and ESD are always kept (can change when these scripts evolve)
# 2. The last processing step is saved (or if different processing steps are connected and the last one is not finished yet
# 3. All .res and .ras files are kept.
# 4. Following steps are never deleted > resampled slave / subt_refdem / filtphase (multilooked) / coherence (multilooked) / unwrap

import os
import sys
from doris.doris_stack.main_code.resdata import ResData

def res_file_selection(path, dat_type='burst'):
    # Give an oversight of all files connected to different steps:

    del_files = []
    res_dat = dict()

    # First check if slave / master / ifgs res files are there
    for res_type in ['master', 'slave', 'ifgs']:
        if os.path.exists(os.path.join(path, res_type + '.res')):
            res_dat[res_type] = ResData(os.path.join(path, res_type + '.res'), type=res_type)
        else:
            print('No data found in ' + path)
            return []

    if res_dat['slave'].process_control['readfiles'] == '1' and res_dat['slave'].process_control['crop'] == '1':
        if res_dat['slave'].processes['readfiles']['deramp'] == '1':
            # Remove the ramped image data.
            del_files.append(res_dat['slave'].processes['crop']['Data_output_file'][:-12] + '.raw')
    if res_dat['ifgs'].process_control['dem_assist'] == '1':
        # Remove the temporary files for dem_assist
        del_files.extend(['dac_delta_demline.temp', 'dac_delta_dempixel.temp', 'dac_m_demline.temp', 'dac_m_dempixel.temp'])
    if res_dat['slave'].process_control['resample'] == '1' and res_dat['ifgs'].process_control['dem_assist'] == '1':
        del_files.extend(['dac_delta_line.raw', 'dac_delta_pixel.raw'])
    if res_dat['slave'].process_control['resample'] == '1':
        # After resampling the deramped file is not needed anymore. Also processing data from resampling is not needed.
        del_files.append(res_dat['slave'].processes['crop']['Data_output_file'])
        del_files.extend(['rsmp_orig_slave_line.raw', 'rsmp_orig_slave_pixel.raw'])

    # Now the resampled slave stays.
    if res_dat['ifgs'].process_control['subtr_refphase'] == '1':
        # If the reference phase is subtracted both the reramped slave and interferogram can be removed
        del_files.extend(['cint.raw', 'slave_rsmp_reramped.raw'])
    if res_dat['ifgs'].process_control['subtr_refdem'] == '1':
        # When the dem phase is removed, the interferogram with subtracted reference phase can be removed.
        del_files.extend(['cint.raw', 'cint_srp.raw', 'demcrop.raw', 'dem_radar.raw', 'master_slave.crd'])
    if res_dat['ifgs'].process_control['filtphase'] == '1':
        # When after the the removal of the reference dem the filtphase step is done, the subtrefdem ifgs is removed.
        del_files.extend(['cint.raw', 'cint_srp.raw', 'cint_srd.raw', 'demcrop.raw', 'dem_radar.raw', 'master_slave.crd'])

    # Finally if it is about a burst image, we can check whether the filtered and coherence files are already
    # concatenated, which means they can be removed.
    image_ifgs = os.path.join(os.path.dirname(os.path.dirname(path)), 'ifgs.res')
    if dat_type == 'burst' and os.path.exists(image_ifgs):
        # If there is a ifgs.res file for the full image.

        image_res = ResData(image_ifgs)
        if image_res.process_control['coherence'] == '1':
            del_files.append('coherence.raw')
        if image_res.process_control['filtphase'] == '1':
            del_files.append('cint.0.2filtered')
        if image_res.process_control['filtphase'] == '1':
            del_files.append('cint.0.2filtered')

    elif dat_type == 'image':
        # If we are looking at the full image, is it important to remove the non-multilooked coherence or filtered image
        if res_dat['ifgs'].process_control['filtphase'] == '1':
            if res_dat['ifgs'].processes['filtphase']['Data_output_file'].endswith('ml.raw'):
                del_files.append('cint.0.2filtered')
        if res_dat['ifgs'].process_control['coherence'] == '1':
            if res_dat['ifgs'].processes['coherence']['Data_output_file'].endswith('ml.raw'):
                del_files.append('coherence.raw')

    del_files = set(del_files)
    del_files = [os.path.join(path, del_file) for del_file in del_files]
    del_files = [del_file for del_file in del_files if os.path.exists(del_file)]

    return del_files

def cleanup_stack(path, master_key):
    # This is the main cleanup function.

    folders = next(os.walk(path))[1]
    if not master_key in folders:
        print('master folder not found in path')
        return
    else:
        folders.remove(master_key)

    del_files = []

    for folder in folders:
        del_dat = res_file_selection(os.path.join(path, folder), dat_type='image')
        del_files.extend(del_dat)

        burst_folders = find_burst_folders(os.path.join(path, folder))
        for burst_folder in burst_folders:
            del_dat = res_file_selection(os.path.join(path, burst_folder), dat_type='burst')
            del_files.extend(del_dat)

    for filename in del_files:
        print('removing: ' + filename)
        os.remove(filename)
    print('Succes! path ' + path + ' is cleaned from temporary files.')


def find_burst_folders(folder):
    # This function finds all the burst folders in an image folder
    folders = []
    swaths = next(os.walk(folder))[1]
    for swath in swaths:
        bursts = next(os.walk(os.path.join(folder, swath)))[1]
        for burst in bursts:
            folders.append(os.path.join(folder, swath, burst))
    return folders

# Actually execute the code...
if __name__ == "__main__":
    path = sys.argv[1]
    master_key = sys.argv[2]

    print('path to be cleaned ' + path)
    print('master key is ' + master_key)

    cleanup_stack(path, master_key)



#!/usr/bin/env python
import os,sys,time

def usage():
    print '\nUsage: python tsx_dump_data.py tsx_COSAR_product outputfile [l0 lN p0 pN] -res RESFILE'
    print '  where tsx_COSAR_product is the input filename'
    print '        outputfile        is the output filename'
    print '        l0                is the first azimuth line (starting at 1)'
    print '        lN                is the last azimuth line'
    print '        p0                is the first range pixel (starting at 1)'
    print '        pN                is the last range pixel'
    print '        RESFILE           DORIS result file that is to be updated for crop metadata'

try:
    inputFileName  = sys.argv[1]; 
    outputFileName = sys.argv[2]
except:
    print 'Unrecognized input'
    usage()
    sys.exit(1)

inputFileName    = sys.argv[1]
outputFileName   = sys.argv[2]

resFile = None
for element in range(len(sys.argv)):
    option = sys.argv[element];
    if option == '-res':
        resFile = str(sys.argv[element+1])
#        print resFile
        del sys.argv[element+1]
        del sys.argv[element]
        break

if len(sys.argv) == 3:
    outputWinFirstLine = None
    outputWinLastLine  = None
    outputWinFirstPix = None
    outputWinLastPix  = None
elif len(sys.argv) > 3 and len(sys.argv) < 9:
    outputWinFirstLine = int(sys.argv[3])-1    # gdal srcwin starting at 0
    outputWinLastLine  = int(sys.argv[4])      # Lastline --> yoff  (later)
    outputWinFirstPix  = int(sys.argv[5])-1    # gdal srcwin starting at 0
    outputWinLastPix   = int(sys.argv[6])      # Lastpix  --> yoff  (later)
elif len(sys.argv) > 3 and len(sys.argv) < 7:
    print 'Unrecognized input'
    usage()
    sys.exit(1)
else:
    outputWinFirstLine = None
    outputWinLastLine  = None
    outputWinFirstPix = None
    outputWinLastPix  = None

# system parameters :: check whether its there
gdalCall = 'gdal_translate'

# Output data parameters
outputDataFormat = 'MFF'
outputDataType   = 'CInt16'

cmd = '%s %s -ot %s -of %s %s' % (gdalCall,inputFileName,outputDataType,outputDataFormat,outputFileName)

if outputWinFirstPix is not None:
    cmd = cmd + (' -srcwin %s %s %s %s' % (outputWinFirstPix,outputWinFirstLine,outputWinLastPix-outputWinFirstPix,outputWinLastLine-outputWinFirstLine))
    #print cmd 

failure = os.system(cmd)
if failure:
    print '%s: running %s failed' % (sys.argv[0],cmd)
    sys.exit(1)
else:
    os.rename(os.path.splitext(outputFileName)[0]+'.j00',outputFileName)
# call to cpxfiddle

# check whether the file exist!!!
if resFile is not None:

    print resFile

    # load header
    headerFileStream = open(os.path.splitext(outputFileName)[0]+'.hdr','r')
    for line in headerFileStream:
        pair = line.split()
        if len(pair) > 1:
            vars()[pair[0]] = pair[2]   # set IMAGE_LINES and LINE_SAMPLES
#            print vars()[pair[0]]

    # check whether the file exist
    outStream = open(resFile,'a')

    outStream.write('\n')
    outStream.write('*******************************************************************\n')
    outStream.write('*_Start_crop:			cosar\n')
    outStream.write('*******************************************************************\n')
    outStream.write('Data_output_file: 	%s\n' % outputFileName)
    outStream.write('Data_output_format: 			complex_short\n') # hardcoded

    if outputWinFirstPix is not None:
        outStream.write('First_line (w.r.t. original_image): 	%s\n' % (outputWinFirstLine+1))   # back to Doris convention
        outStream.write('Last_line (w.r.t. original_image): 	%s\n' % outputWinLastLine)
        outStream.write('First_pixel (w.r.t. original_image): 	%s\n' % (outputWinFirstPix+1))
        outStream.write('Last_pixel (w.r.t. original_image): 	%s\n' % outputWinLastPix)

    elif outputWinFirstPix is None:
        outStream.write('First_line (w.r.t. original_image): 	1\n')
        outStream.write('Last_line (w.r.t. original_image): 	%s\n' % IMAGE_LINES)
        outStream.write('First_pixel (w.r.t. original_image): 	1\n')
        outStream.write('Last_pixel (w.r.t. original_image): 	%s\n' % LINE_SAMPLES)

    outStream.write('*******************************************************************\n')
    outStream.write('* End_crop:_NORMAL\n')
    outStream.write('*******************************************************************\n')

    outStream.write('\n')
    outStream.write('    Current time: %s\n' % time.asctime())
    outStream.write('\n')

    # close output file
    outStream.close()

    # replace crop tag in result file
    sourceText   = "crop:			0"
    replaceText  = "crop:			1"
    inputStream  = open(resFile,'r')
    textStream   = inputStream.read()
    inputStream.close()
    outputStream = open(resFile, "w")
    outputStream.write(textStream.replace(sourceText, replaceText))
    outputStream.close()



import os
import time

class Jobs(object):
    """The Jobs class runs a list of jobs in parallel.
       It starts the maximum number of jobs from the list in parallel and monitors for job completion.
       When jobs are finished, new jobs are started until the maximum is reached again.
       This is repeated until all jobs from the list are processed.

       This class executes a jobHandlerScript to execute a job.
       The job handler script sets job start and job finished flags in a flag directory on the system.
       In verbose mode the job handler script prints status info to stdout

       This class generates directories on the system that contain start and finish flags for each job that is run
       Old flag directories are moved to timestamped directories

       Methods:
           Run: executes list of jobs
           """
    def __init__(self, max_jobs, dorisParameters):
        """max_jobs: maximum number of jobs to run simultaniously
           verbose: print status to stdout during execution of job list"""
        self.max_jobs = max_jobs
        self.pid = str(os.getpid())
        self.doris_parameters = dorisParameters
        self.verbose = self.doris_parameters.verbose
        self.flag_dir_root = self.doris_parameters.doris_parallel_flag_dir
        self.between_sleep_time = self.doris_parameters.between_sleep_time
        self.end_sleep_time = self.doris_parameters.end_sleep_time
        self.python_path = os.path.dirname(self.doris_parameters.source_path)
        self.jobs_todo = []
        self.jobs_active = []
        self.jobs_finished = []
        self.flag_dir = ''

    def _new_id(self):
        # static class variable
        Jobs.id = Jobs.id + 1
        return Jobs.id

    def _set_id(self, job_list):
        for job_dict in job_list:
            job_dict['id'] = self._new_id()

    def _create_flag_dir(self):
        self.flag_dir = self.flag_dir_root + "." + time.asctime(time.localtime(time.time())).replace(" ", "_")
        os.mkdir(self.flag_dir)


    def _cleanup_flag_dir(self):
        #
        # cleans stuff from current run, if not verbose
        #
        if(not (self.verbose)):
            os.system("rm -rf " + self.flag_dir)

    def _get_job_id(self, job):
        #
        # returns machine level unique job Id
        return job['path'].replace("/","_") + "." + job['command'].replace("/","_").replace(" ","-") + self.pid


    def _start_jobs(self):
        #
        # starts a number of jobs
        # returns list of unstarted and list of started jobs
        #
        jobs_to_start_count = min((self.max_jobs - len(self.jobs_active)), len(self.jobs_todo))
        for index in range(0, jobs_to_start_count):
            job = self.jobs_todo.pop(0)
            os.chdir(job['path'])
            os.system(self.doris_parameters.job_handler_script + " "
                      + self.flag_dir + " "
                      + str(job['id']) + " "
                      + self._get_job_id(job) + " "
                      + str(self.verbose) + " "
                      + self.python_path + " "
                      + job['command'] + " &")
            self.jobs_active.append(job)
        return

    def _check_active_jobs(self):
        #
        # returns from the list of jobs, the jobs that are started, but not finished
        #
        jobs_active = []
        for job in self.jobs_active:  # find active jobs
            this_job_started = False
            this_job_ready = False
            for file in os.listdir(self.flag_dir):
                if str(job['id']) + ".finished" == file:
                    this_job_ready = True
#                if str(job['id']) + ".started" == file:
#                    this_job_started = True
#            this_job_active = this_job_started & (not (this_job_ready))
#            if (this_job_active):
            if (not (this_job_ready)):
                jobs_active.append(job)
        self.jobs_active = jobs_active
        return

    def run(self, job_list):
        """executes joblist in parallel
        jobList: list of jobs, containing execution path and command to be executed
        """
        self._create_flag_dir()
        self.jobs_todo = job_list
        self._set_id(self.jobs_todo)
        self._start_jobs()
        while len(self.jobs_active):
            if(self.verbose):
                print time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.gmtime()) + "jobs busy"
            time.sleep(self.between_sleep_time)
            self._check_active_jobs()
            self._start_jobs()
        if (self.verbose):
            print time.strftime("%a, %d %b %Y %H:%M:%S +0000", time.gmtime()) + "jobs finished"
        time.sleep(self.end_sleep_time)
        self._cleanup_flag_dir()



# This files defines a class for metadata objects of sentinel images. Large part of the work depends on python readers
# from the tops toolbox.
import os
import warnings
import zipfile
import copy

from doris.doris_stack.main_code.swath import SwathMeta


class ImageMeta(object):
    # Object for image files for sentinel data

    def __init__(self, path='' , pol='all' ,swath_no=['1','2','3']):
        # Initialize function variables

        # This will contain a list of swath objects
        self.swaths = []
        self.pol = pol
        self.swath_no = swath_no

        # The following contain the path of xml and data files
        self.swaths_xml = []
        self.swaths_data = []
        self.image_kml = ''

        # This variable contains the convex hull of all swaths together
        self.metadata = []
        self.coverage = []

        # The following variables store data on which processing steps are performed and the results of these steps.
        self.steps = []
        self.steps_res = []

        # The following variable is to check the total number of bursts for this image. This is used to remove time
        # slots with less bursts.
        self.burst_no = 0

        # Check if the data is unzipped or not. If unzipped run the further initialization.
        self.zip_path = ''
        self.unzip_path = ''
        if path.endswith('.zip'):
            self.zip_path = path
        else:
            self.unzip_path = path

        # orbit information for this image
        self.orbit = ''

    ######################################################

    def init_unzipped(self, unzip_path=''):
        # This function creates an image object and searches for available data and xml files. It gives an error when
        # either the path does not exist, no data or xml files can be found or the data and xml files do not match.
        # It is possible to choose one of the available polarisations or swaths using the pol and swath variables.
        xml_dir = os.path.join(self.unzip_path, 'annotation')
        xml = [f for f in os.listdir(xml_dir) if os.path.isfile(os.path.join(xml_dir, f))]

        data_dir = os.path.join(self.unzip_path, 'measurement')
        data = [f for f in os.listdir(data_dir) if os.path.isfile(os.path.join(data_dir, f))]

        # Select polarisation
        if any(s in self.pol for s in ('hh','vv','hv','vh')):
            xml = [x for x in xml if x[12:14] in self.pol]
            data = [x for x in data if x[12:14] in self.pol]
        elif self.pol != 'all':
            warnings.warn('Polarisation not recognized, using default (all)')

        # Select swaths
        xml = sorted([os.path.join(self.unzip_path,'annotation',x) for x in xml if x[6] in self.swath_no])
        data = sorted([os.path.join(self.unzip_path,'measurement',x) for x in data if x[6] in self.swath_no])

        # Initialize function values
        dat = [os.path.basename(d) for d in data]
        self.swaths_xml = [x for x in xml if os.path.basename(x)[:-4] + '.tiff' in dat]
        self.swaths_data = data

        # Check if the data is there and if the filenames coincide.
        if len(self.swaths_xml) == 0:
            warnings.warn('There are no xml files')


    def unzip(self, unzip_path=''):
        # This function unzips the corresponding image, based on some requirements.
        # Note that this is a backup function, while most unpacking is done by load_shape_unzip.py
        if not os.path.exists(self.path):
            try:
                zip = zipfile.ZipFile(self.path + '.zip')
                path = os.path.abspath(os.path.join(self.path, os.pardir))
                zip.extractall(path)
                return True
            except:
                print('Failed to unpack!')
                return False
        else:
            return True

    def meta_swath(self, precise_folder=''):
        # This function reads and stores metadata of different swaths in the swath objects.
        orbits = []
        orb_type = ''

        if not self.swaths_data:
            self.init_unzipped()

        if not self.swaths:
            for i in range(len(self.swaths_data)):
                data = self.swaths_data[i]
                xml = os.path.join(os.path.dirname(os.path.dirname(data)), 'annotation', os.path.basename(data)[:-5] + '.xml')

                # Initialize swath and load data from xml file
                swath = SwathMeta(xml=xml, data=data)
                swath.meta_swath()

                # Calculate the orbits for this swath and reuse it for other swaths if it is already calculated
                if not orbits or not orb_type:
                    orbits, orb_type = swath.orbits_swath(precise_folder=precise_folder)
                else:
                    swath.orbits = copy.deepcopy(orbits)
                    swath.orbit_type = orb_type

                # Define the resdata for the individual burst. And append the swath to the image object.
                swath.meta_burst()
                self.swaths.append(swath)



#!/usr/bin/env python

#-----------------------------------------------------------------#
# A python code for extracting image matrix from RS2 geotiff file 
# using gdal_translate
#
# Author: TUDelft - 2010
# Maintainer: Mahmut Arikan
#
# Developed based on tsx_dump_data.py code.
#
#-----------------------------------------------------------------#
import os,sys,time
from os import path

import xml.etree.ElementTree as etree  # parameters required for cropping and flipping


codeRevision=1.1   # this code revision number
def usage():
    print 'INFO    : @(#)Doris InSAR software, $Revision: %s $, $Author: TUDelft $' % codeRevision
    print
    print 'Usage   : rs2_dump_data.py <inputfile> <outputfile> [l0 lN p0 pN] [-res RESFILE]'
    print 
    print '          inputfile         is the input Radarsat-2 geotiff filename : master.tif'
    print '          outputfile        is the output filename                   : master.slc'
    print '          l0                is the first azimuth line (starting at 1)'
    print '          lN                is the last azimuth line'
    print '          p0                is the first range pixel (starting at 1)'
    print '          pN                is the last range pixel'
    print '          RESFILE           DORIS result file that is to be updated for crop metadata (optional)'
    print
    print '          This software is part of Doris InSAR software package.\n'
    print '(c) 1999-2010 Delft University of Technology, the Netherlands.\n'

try:
    inputFileName  = sys.argv[1] 
    outputFileName = sys.argv[2]
except:
    print '\nError   : Unrecognized input or missing arguments\n\n'
    usage()
    sys.exit(1)

inputFileName    = sys.argv[1]
outputFileName   = sys.argv[2]

resFile = None
for element in range(len(sys.argv)):
    option = sys.argv[element];
    if option == '-res':
        resFile = str(sys.argv[element+1])
#        print resFile
        del sys.argv[element+1]
        del sys.argv[element]
        break


if len(sys.argv) == 3:
    outputWinFirstLine = None
    outputWinLastLine  = None
    outputWinFirstPix = None
    outputWinLastPix  = None
elif len(sys.argv) > 3 and len(sys.argv) < 9:
    outputWinFirstLine = int(sys.argv[3])    # Firstline
    outputWinLastLine  = int(sys.argv[4])    # Lastline
    outputWinFirstPix  = int(sys.argv[5])    # Firstpix
    outputWinLastPix   = int(sys.argv[6])    # Lastpix
elif len(sys.argv) > 3 and len(sys.argv) < 7:
    print '\nError   : Unrecognized input or missing arguments\n\n'
    usage()
    sys.exit(1)
else:
    outputWinFirstLine = None
    outputWinLastLine  = None
    outputWinFirstPix = None
    outputWinLastPix  = None



# Extract some important parameters for coordinate manipulation
productfile  = path.join(path.dirname(inputFileName), 'product.xml')
tree         = etree.parse(productfile) # 
NS           = 'http://www.rsi.ca/rs2/prod/xml/schemas'
PASS         = tree.find('.//{%s}passDirection'     % NS).text     # Ascending or Descending
line_order   = tree.find('.//{%s}lineTimeOrdering'  % NS).text
pixl_order   = tree.find('.//{%s}pixelTimeOrdering' % NS).text # Increasing or Decreasing
Nlines       = int(tree.find('.//{%s}numberOfLines'          % NS).text)
Npixls       = int(tree.find('.//{%s}numberOfSamplesPerLine' % NS).text)
in_FirstLine = outputWinFirstLine
in_LastLine  = outputWinLastLine
in_FirstPix  = outputWinFirstPix
in_LastPix   = outputWinLastPix

if line_order == 'Increasing':
    print('INFO     : Detected a imagery %s line time order'  % line_order) 
else:
    print('INFO     : Detected a imagery %s line time order'  % line_order) 
    print('INFO     : Adjusting first line and last line ') 
    outputWinFirstLine = Nlines-in_LastLine+1
    outputWinLastLine  = Nlines-in_FirstLine+1
    print Nlines, Npixls
    print outputWinFirstLine,outputWinLastLine,outputWinFirstPix,outputWinLastPix

if pixl_order == 'Increasing':
    print('INFO     : Detected a imagery %s pixel time order' % pixl_order) 
else:
    print('INFO     : Detected a imagery %s pixel time order' % pixl_order) 
    print('INFO     : Adjusting first pixel and last pixel ') 
    outputWinFirstPix = Npixls-in_LastPix+1
    outputWinLastPix  = Npixls-in_FirstPix+1
    print Nlines, Npixls
    print outputWinFirstLine,outputWinLastLine,outputWinFirstPix,outputWinLastPix


# GDAL Extract image matrix using gdal_translate
# Ex: gdal_translate -of ENVI -ot Int16 -co INTERLEAVE=BIP imagery_HH.tif rs2_slc.raw
gdalCall = 'gdal_translate'

# GDAL parameters
outputDataFormat = 'ENVI'
#outputDataType   = 'CInt16'
outputDataType   = 'Int16'
outputDataType2   = 'ci2'  # for cpxfiddle
outputInterleave = 'INTERLEAVE=BIP'

cmd = '%s %s -ot %s -of %s -co %s %s' % (gdalCall,inputFileName,outputDataType,outputDataFormat,outputInterleave,outputFileName+'.noflip')

if outputWinFirstPix is not None:
    #                                         xoff                  yoff                  xsize = width                     ysize= height
    #                                        1 --> 0               1 --> 0
    cmd = cmd + (' -srcwin %s %s %s %s' % (outputWinFirstPix-1,outputWinFirstLine-1,outputWinLastPix-outputWinFirstPix+1,outputWinLastLine-outputWinFirstLine+1))
    print cmd 

failure = os.system(cmd)
if failure:
    print '%s: running %s failed' % (sys.argv[0],cmd)
    sys.exit(1)
#else:
#    os.rename(os.path.splitext(outputFileName)[0]+'.j00',outputFileName)



print('\nINFO     : Start flipping image to radar coordinate for a %s orbit.' % PASS)

# Flip using cpxfiddle
# get keyword asc or desc from .res
#  if resFile is not None:
#     inStream = open(resFile,'r')
#     find ascending tag then
# -m Y for ascending
# -m X for descending orbit

cpxfcall='cpxfiddle -q normal -o short '              # cpxfiddle -w 3788 -f ci2 -q normal -o short -m Y rs2_slc.raw.noflip  > rs2_slc.raw 

if PASS == 'Ascending':
    cmd = cpxfcall + ( '-w %s -f %s -m %s %s > %s ' % (outputWinLastPix-outputWinFirstPix+1,outputDataType2,'Y',outputFileName+'.noflip',outputFileName)) 
    print('INFO     : %s ' % cmd) 
else:
    cmd = cpxfcall + ( '-w %s -f %s -m %s %s > %s ' % (outputWinLastPix-outputWinFirstPix+1,outputDataType2,'X',outputFileName+'.noflip',outputFileName)) 
    print('INFO     : %s ' % cmd) 

failure = os.system(cmd)
if failure:
    print '%s: running %s failed' % (sys.argv[0],cmd)
    sys.exit(1)
else:
    os.remove(outputFileName+'.noflip')



# Manual call to update Doris res file
# check whether the file exist!!!
if resFile is not None:

    print resFile

    # load header
    #headerFileStream = open(os.path.splitext(outputFileName)[0]+'.hdr','r')
    headerFileStream = open(outputFileName+'.hdr','r')
    for line in headerFileStream:
        pair = line.split()
        if len(pair) > 1:
            vars()[pair[0]] = pair[2]   # set IMAGE_LINES and LINE_SAMPLES
#            print vars()[pair[0]]

    # check whether the file exist
    outStream = open(resFile,'a')

    outStream.write('\n')
    outStream.write('*******************************************************************\n')
    outStream.write('*_Start_crop:			geotiff\n')
    outStream.write('*******************************************************************\n')
    outStream.write('Data_output_file:                       %s\n' % outputFileName)
    outStream.write('Data_output_format: 			complex_short\n') # HARDCODED

    if outputWinFirstPix is not None:
        outStream.write('First_line (w.r.t. original_image): 	%s\n'   % in_FirstLine)   
        outStream.write('Last_line (w.r.t. original_image): 	%s\n'   % in_LastLine)
        outStream.write('First_pixel (w.r.t. original_image): 	%s\n' % in_FirstPix)
        outStream.write('Last_pixel (w.r.t. original_image): 	%s\n'   % in_LastPix)

    elif outputWinFirstPix is None:
        outStream.write('First_line (w.r.t. original_image): 	1\n')
        outStream.write('Last_line (w.r.t. original_image): 	%s\n'   % lines )
        outStream.write('First_pixel (w.r.t. original_image): 	1\n')
        outStream.write('Last_pixel (w.r.t. original_image): 	%s\n'   % samples )

    outStream.write('*******************************************************************\n')
    outStream.write('* End_crop:_NORMAL\n')
    outStream.write('*******************************************************************\n')

    outStream.write('\n')
    outStream.write('    Current time: %s\n' % time.asctime())
    outStream.write('\n')

    # close output file
    outStream.close()

    # replace crop tag in result file
    sourceText   = "crop:			0"
    replaceText  = "crop:			1"
    inputStream  = open(resFile,'r')
    textStream   = inputStream.read()
    inputStream.close()
    outputStream = open(resFile, "w")
    outputStream.write(textStream.replace(sourceText, replaceText))
    outputStream.close()


#EOF



import argparse
from prepare_datastack import PrepareDatastack

"""Doris prepare datastack
arguments:  --doris_input_file, -i
"""

# parse arguments here

parser = argparse.ArgumentParser(description='Doris prepare datastack.')
parser.add_argument('--doris_input_file', '-i', default='',
                    help='Path to doris input file, this file contains case specific parameters')

args = parser.parse_args()

#start doris sentinel1 run
prepare_data_stack = PrepareDatastack()
prepare_data_stack.prepare(args.doris_input_file)





# This script gathers available sentinel files from the database and checks the coverage in space and time for this data
# stack.

import os
import warnings
from collections import Counter, OrderedDict
from datetime import datetime
import shutil

import fiona
import numpy as np
from shapely.geometry import shape, mapping, box
from shapely.ops import cascaded_union

import image as image
from doris.doris_stack.functions.load_shape_unzip import extract_kml_preview, shape_im_kml, load_shape
from doris.doris_stack.main_code.dorisparameters import DorisParameters
from doris.doris_stack.functions.burst_metadata import center_shape_from_res
from doris.doris_stack.main_code.jobs import Jobs


class StackData(object):
    # This function holds information for a full datastack of sentinel data and is used to select relevant images and
    # bursts based on a given area of interest.

    def __init__(self, track_dir, shape_dat, buffer=0.02, start_date='2014-04-01', end_date='', polarisation='vh', path='', db_type=1, precise_dir=''):
        # Initialize variables:

        Jobs.id = 0

        # Datastack folder where all data from the datastack is stored (database,shapes,burst data, res files and results)
        # You should have write acces to this folder!
        self.path = path
        self.unzip_path = ''

        # Search path, shape, buffer and polarisation for this datastack. Currently only one polarisation is implemented. If
        # needed this can be extended later.
        self.search_path = []
        self.start_date = []
        self.end_date = []
        self.master_date = []
        self.shape = []
        self.shape_filename = shape_dat
        self.buffer = []
        self.polarisation = ''
        self.precise_orbits = ''

        # All images included in this datastack. Including lists of shapes and acquistion dates
        self.images = []
        self.image_files = []
        self.image_shapes = []
        self.image_dates = []

        # The shapes and names of the swaths
        self.swath_names = list()
        self.swath_shapes = list()

        # The resulting dates, with underlying bursts. This information can be used to create res files and images
        # of individual bursts. Variable data includes a structure using (dates > swaths > bursts).
        self.dates = list()
        self.datastack = dict()
        self.concatenated = dict()
        self.coordinates = dict()
        self.burst_names = list()
        self.burst_shapes = list()
        self.burst_centers = list()

        # Some important characteristics of the dataset of bursts. (matrices with names on y and dates on x axis)
        self.burst_no = 0
        self.burst_availability = []
        self.burst_lon = []
        self.burst_lat = []
        self.burst_baselines = []

        # Temporary variable to store images which are not yet checked for shape or dates.
        self.image_dump = []

        # parallel computing:
        doris_parameters = DorisParameters(os.path.dirname(self.path))
        self.doris_parameters = doris_parameters
        self.nr_of_jobs = doris_parameters.nr_of_jobs
        self.parallel = doris_parameters.parallel
        self.function_path = doris_parameters.function_path

        ####################################################################

        # This function initializes the datastack using a search path, start/end dates and a buffer shape. The start and
        # end date should have the format yyyy-mm-dd and the shape should either be a shapefile or a list of coordinate
        # pairs. [[lat,lon][lat,lon] enz. ]. Minimum input is a folder which contains the .SAFE folders (a track folder)

        if not track_dir or not os.path.exists(track_dir):
            warnings.warn('This function needs an existing path as input!')
            return

        self.search_files(track_dir)

        if shape:
            self.create_shape(shape_dat,buffer)

        if end_date:
            self.end_date = np.datetime64(end_date).astype('datetime64[s]') + np.timedelta64(1, 'D').astype('timedelta64[s]')
        else:
            self.end_date = np.datetime64('now').astype('datetime64[s]')
        self.start_date = np.datetime64(start_date).astype('datetime64[s]')

        if isinstance(polarisation, basestring):
            polarisation = [polarisation]
            for i in polarisation:
                if not i in ['hh','vv','hv','vh']:
                    warnings.warn('This polarisation does not exist for sentinel data.')
                    return
        self.polarisation = polarisation
        self.search_path = track_dir

        if not path:
            warnings.warn('You did not specify an output path. Please do so later on using the add_path function')
        else:
            self.add_path(path)

        if precise_dir:
            if os.path.exists(precise_dir):
                self.precise_orbits = precise_dir
            else:
                print 'Precise orbit path does not exist'

    def add_path(self,path):
        # This function adds the output path.

        if os.path.isdir(path):
            self.path = path
        elif os.path.isdir(os.path.dirname(path[:-1])):
            os.mkdir(path)
            self.path = path
        else:
            warnings.warn('Neither the directory itself nor the parents directory exists. Choose another path.')

    def search_files(self, track_dir):
        # This function searches for files within a certain folder. This folder should contain images from the same
        # track. These images are added to the variable image dump.
        images = list()

        top_dir = next(os.walk(track_dir))

        for data in top_dir[2]:
            if data.endswith('.zip'):
                images.append(os.path.join(track_dir, data))
        for data in top_dir[1]:
            if data.endswith('.SAFE'):
                images.append(os.path.join(track_dir, data))
            else:  # Likely images are stored in a folder.
                sec_dir = next(os.walk(os.path.join(track_dir, data)))

                for dat in sec_dir[1]:
                    if dat.endswith('.SAFE'):
                        images.append(os.path.join(track_dir, data, dat))
                for dat in sec_dir[2]:
                    if dat.endswith('.zip'):
                        images.append(os.path.join(track_dir, data, dat))

        if images:
            images = sorted(images)
        else:
            print('No images found in track_dir = ' + str(track_dir) + ', switching to archive folder')

            track_name = self.doris_parameters.direction + '_t' + self.doris_parameters.track.zfill(3)
            track_dir = os.path.join(track_dir, track_name)

            print('Searching in folder ' + track_dir)

            top_dirs = next(os.walk(track_dir))[1]

            if len(top_dirs) == 1:
                track_dir = os.path.join(track_dir, top_dirs[0])
            elif len(top_dirs) > 1:
                for top_dir in top_dirs:
                    user_input = raw_input("Do you want to use folder " + top_dir + " as resource folder? (yes/no)").lower()
                    if user_input in ['yes', 'y']:
                        track_dir = os.path.join(track_dir, top_dir)

            dir_new = next(os.walk(track_dir))
            for dat in dir_new[1]:
                dat_files = next(os.walk(os.path.join(track_dir, dat)))[2]
                for data in dat_files:
                    if data.endswith('.zip'):
                        images.append(os.path.join(track_dir, dat, data))

        if images:
            images = sorted(images)
        else:
            warnings.warn('No images found! Please choose another data folder. Track_dir = ' + str(track_dir))

        base = []  # Remove double hits because data is already unzipped.
        for i in images: # Drop all .zip files which are unpacked already.
            if i.endswith('.SAFE.zip'):
                base.append(os.path.basename(i[:-9]))
            elif i.endswith('.zip'):
                base.append(os.path.basename(i[:-4]))
            elif i.endswith('.SAFE'):
                base.append(os.path.basename(i[:-5]))
        b, id = np.unique(base, return_index=True)

        rem = []
        for i in range(len(base)):
            if i in id:
                self.image_dump.append(images[i])
            else:
                rem.append(images[i])

        if rem:
            print('removed the following zip files from stack:')
            for r in rem:
                print(r)
            print('It is advised to work with zipfiles instead of unpacked data. This saves diskspace and will ')

    def create_shape(self,shape_dat,buffer=0.02):
        # This function creates a shape to make a selection of usable bursts later on. Buffer around shape is in
        # degrees.

        self.shape = load_shape(shape_dat, buffer)
        self.buffer = buffer

    def check_new_images(self, master):
        # This function checks which images are already processed, and which are not. If certain dates are already
        # processed they are removed from the list. You have to specify the master date, otherwise the script will not
        # know how many burst are expected per date.

        # Which dates are available?
        image_dates = [im.astype('datetime64[D]') for im in self.image_dates]
        # What is the master date
        date = np.datetime64(master).astype('datetime64[D]')

        date_folders = [d for d in next(os.walk(self.path))[1] if len(d) == 8]
        rm_id = []

        if date_folders:
            dates = [np.datetime64(d[0:4] + '-' + d[4:6] + '-' + d[6:8]) for d in date_folders]

            if date in dates:
                date_folder = date_folders[np.where(dates == date)[0][0]]

                # Check existing files in master folder
                swaths = dict()
                swath_folders = next(os.walk(os.path.join(self.path, date_folder)))[1]
                if len(swath_folders) == 0:
                    print('No swaths in master folder')
                    return

                for swath in swath_folders:
                    self.swath_names.append(swath)
                    swaths[swath] = sorted(next(os.walk(os.path.join(self.path, date_folder, swath)))[1])

                # Now check if the burst also in slave folders exist....
                for folder, d in zip(date_folders, dates):
                    # Check existing files in master folder
                    try:
                        swath_folders = next(os.walk(os.path.join(self.path, folder)))[1]
                        if not set(swath_folders) == set(swaths.keys()):
                            raise LookupError('Amount of swaths is not the same for ' + folder)

                        for swath in swath_folders:
                            bursts = sorted(next(os.walk(os.path.join(self.path, date_folder, swath)))[1])
                            if not set(bursts) == set(swaths[swath]):
                                raise LookupError('Amount of bursts is not the same for ' + folder)

                            if d == date:
                                # If the master is already processed we have to create the list of center and coverage of
                                # bursts.
                                # TODO make this robust for the case no seperate input data folders are created.
                                res_files = []
                                for burst in bursts:
                                    dat_file = [r for r in next(os.walk(os.path.join(self.path, date_folder, swath, burst)))[2]
                                                if (r.startswith('slave_iw') and len(r) < 25)]
                                    res_file = os.path.join(self.path, date_folder, swath, burst, 'slave.res')
                                    res_files.append(res_file)

                                    center, coverage = center_shape_from_res(resfile=res_file)
                                    # Assign coverage, center coordinates and burst name.
                                    self.burst_shapes.append(coverage)
                                    self.burst_centers.append(center)
                                    burst_num = os.path.basename(dat_file[0])[17:-4]
                                    self.burst_names.append(swath + '_burst_' + burst_num)
                                    self.burst_no += 1

                        # If all bursts are the same these files are not processed.
                        for id in np.where(image_dates == d)[0][::-1]:
                            del self.image_dates[id]
                            del image_dates[id]
                            del self.images[id]
                            del self.image_files[id]

                    except LookupError as error:
                        print(error)

    def select_image(self,start_date='',end_date='', dest_folder=''):
        # This function selects usable images based on .kml files and dates

        if not dest_folder:
            dest_folder = os.path.join(self.path, 'kml')
        if not os.path.exists(dest_folder):
            os.makedirs(dest_folder)

        if not self.shape:
            warnings.warn('There is no shape loaded to select images. Please use the create_shape function to do so.')

        if start_date:
            self.start_date = np.datetime64(start_date).astype('datetime64[s]')
        if end_date:
            self.end_date = np.datetime64(end_date).astype('datetime64[s]') + np.timedelta64(1, 'D').astype('timedelta64[s]')

        # First select images based on dates and check if polygons intersect.
        for i in self.image_dump:
            d = os.path.basename(i)[17:32]
            acq_time = np.datetime64(d[0:4] + '-' + d[4:6] + '-' + d[6:11] + ':' + d[11:13] + ':' + d[13:] + '-0000')
            if acq_time >= self.start_date and acq_time <= self.end_date:
                im = image.ImageMeta(path=i)
                kml, png = extract_kml_preview(i, dir=dest_folder, png=False, overwrite=True)
                succes = shape_im_kml(self.shape, kml)

                if succes:
                    self.images.append(im)
                    self.image_dates.append(acq_time)
                    self.image_files.append(os.path.basename(i))
        self.dates = sorted(list(set([d.astype('datetime64[D]') for d in self.image_dates])))
        for date in self.dates:
            print(date)

    def select_burst(self, date=''):
        # This function selects the usefull bursts at one epoch (user defined or automatically selected) and searches
        # usefull burst at other dates. This function uses the extend_burst function, which is intended to search for
        # bursts at other dates. This function can be run later on to update the datastack.

        if self.burst_names:
            print('Master data is already loaded from earlier processed data.')
            return

        image_dates = [im.astype('datetime64[D]') for im in self.image_dates]
        # First select which date will be the master
        if date:
            date = np.datetime64(date).astype('datetime64[D]')
            # date = deepcopy(image_dates[min(abs(self.image_dates-date))])
        else:  # if no date is specified
            date = Counter(image_dates).most_common(1)[0][0]

        # Load the metadata for this date for the bursts and swaths
        for date in image_dates:
            print(date)
        print(date)
        image_id = np.where(image_dates == date)[0]
        for i in image_id:
            print('processing data: ' + self.images[i].unzip_path)
            self.images[i].meta_swath(precise_folder=self.precise_orbits)

        # Order the selected images by acquisition time.
        image_id = [x for (y,x) in sorted(zip([self.image_dates[i] for i in image_id],image_id))]
        date = date.astype(datetime).strftime('%Y-%m-%d')
        self.master_date = date
        self.datastack[date] = dict()

        swath_nums = ['1', '2', '3']

        for p in self.polarisation:
            for swath in swath_nums:
                swath_id = 'iw' + swath + '-slc-' + p
                swath_name = 'swath_' + swath
                burst_no = 1
                data = []

                for i in image_id:
                    # Check which swath should be selected.
                    if not self.images[i].swaths_data:
                        continue

                    swath_names = [os.path.basename(data) for data in self.images[i].swaths_data]
                    swath_no = [no for no in range(len(swath_names)) if swath_id in swath_names[no]]
                    if not swath_no:  # If there is no data for this swath
                        continue

                    swath_no = swath_no[0]

                    for burst in self.images[i].swaths[swath_no].bursts:
                        if burst.burst_coverage.intersects(self.shape):

                            # Check if we do not have a copy of former bursts...
                            if len(self.burst_centers) != 0:
                                dist_center = [np.sqrt((burst.burst_center[0] - center[0])**2 +
                                               (burst.burst_center[1] - center[1])**2) for center in self.burst_centers]
                                if min(dist_center) < 0.02:
                                    print('Possible duplicate burst deleted')
                                    continue

                            # If there are bursts in this swath, which intersect, add burst to list.
                            if swath_name not in self.datastack[date].keys(): # If there are burst and no burst list exists
                                self.datastack[date][swath_name] = dict()
                                if swath_name not in self.swath_names:
                                    self.swath_names.append(swath_name)

                            # Assign burst to data stack
                            burst_name = 'burst_' + str(burst_no)
                            burst.new_burst_num = burst_no
                            self.datastack[date][swath_name][burst_name] = burst

                            # Create .res file data
                            burst.meta_burst(swath_meta=self.images[i].swaths[swath_no].metadata)

                            # Assign coverage, center coordinates and burst name.
                            self.burst_shapes.append(burst.burst_coverage)
                            self.burst_centers.append(burst.burst_center)
                            self.burst_names.append(swath_name + '_' + burst_name)
                            burst_no += 1

                            # Finally add also to the number of bursts from the image
                            self.images[i].burst_no += 1
                            self.burst_no += 1

    def extend_burst(self):
        # Searches for burst at dates other than the dates that are already available. This means that if there are
        # images added for dates which are already indexed, this data will not be used!

        image_dates = [d.astype('datetime64[D]') for d in self.image_dates]

        for date in self.dates:
            # Append data to datastack variable
            date_str = date.astype(datetime).strftime('%Y-%m-%d')
            self.datastack[date_str] = dict()
            data = []

            # Load the metadata for this date for the bursts and swaths
            image_id = np.where(image_dates == date)[0]
            if len(image_id) == 0:
                continue

            for i in image_id:
                if date_str == self.master_date:
                    continue
                print('processing data: ' + self.images[i].unzip_path)
                self.images[i].meta_swath(precise_folder=self.precise_orbits)

            for swath in self.swath_names:
                # Add swath to datastack
                self.datastack[date_str][swath] = OrderedDict()

                for i in image_id:
                    # Select correct swath in image
                    swath_id = swath[-1] + '-slc-' + self.polarisation[0]

                    swath_names = [os.path.basename(data) for data in self.images[i].swaths_data]
                    swath_no = [no for no in range(len(swath_names)) if swath_id in swath_names[no]]
                    if not swath_no:  # If there is no data for this swath
                        continue

                    swath_no = swath_no[0]

                    for burst in self.images[i].swaths[swath_no].bursts:
                        x_dist = np.array([xy[0] - burst.burst_center[0] for xy in self.burst_centers])
                        y_dist = np.array([xy[1] - burst.burst_center[1] for xy in self.burst_centers])

                        dist = np.sqrt(x_dist**2 + y_dist**2)
                        burst_id = np.argmin(dist)

                        if dist[burst_id] < 0.1:
                            # Assign burst to data stack
                            burst.new_burst_num = int(self.burst_names[burst_id][14:])
                            self.datastack[date_str][swath][self.burst_names[burst_id][8:]] = burst

                            # Create .res file data
                            burst.meta_burst(swath_meta=self.images[i].swaths[swath_no].metadata)
                        else:
                            print('No corresponding burst found! Closest is ' + str(dist[burst_id]) + ' ' +
                                  date_str + ' ' + swath + ' ' + self.burst_names[burst_id])

        # Remove all unnecessary dates from stack
        for dat_key in self.datastack.keys():
            if not self.datastack[dat_key]:
                del self.datastack[dat_key]

    def remove_incomplete_images(self):
        # This function removes all the images with less than maximum bursts. This will make a stack more consistent.

        for key in self.datastack.keys():
            burst_no = 0
            for key_swath in self.datastack[key].keys():
                burst_no += len(self.datastack[key][key_swath])
            if burst_no != self.burst_no:
                self.datastack.pop(key)
                print('Number of burst for ' + key + ' is ' + str(burst_no) + ' instead of ' + str(self.burst_no) +
                      ' and is removed from the datastack.')

    def define_burst_coordinates(self,slaves=False):
        # This function defines the exact coordinates in pixels of every burst based on the lower left corner of the first
        # burst image. In this way the total overlap of these bursts can easily be monitored. Results are written to the
        # coordinates variable

        if slaves is True:
            dates = self.datastack.keys()
        else:
            dates = [self.master_date]

        self.coordinates = OrderedDict()

        for date in dates:
            self.coordinates[date] = OrderedDict()
            self.coordinates[date]['shapes'] = []
            ref = False

            min_line = 1; min_pixel = 1
            max_line = 1; max_pixel = 1
            for swath in self.datastack[date].keys():

                self.coordinates[date][swath] = OrderedDict()
                self.coordinates[date][swath]['corners'] = np.zeros([len(self.datastack[date][swath].keys()), 4, 2],dtype='int')

                b = 0
                for burst in sorted(self.datastack[date][swath].keys(), key = lambda x: int(x[6:])):

                    if ref is False:
                        self.coordinates[date]['ref_az_time'] = self.datastack[date][swath][burst].processes['readfiles']['First_pixel_azimuth_time (UTC)']
                        self.coordinates[date]['ref_range_time'] = self.datastack[date][swath][burst].processes['readfiles']['Range_time_to_first_pixel (2way) (ms)']
                        ref = True

                    az_first = self.datastack[date][swath][burst].processes['readfiles']['First_pixel_azimuth_time (UTC)']
                    az_samp = self.datastack[date][swath][burst].processes['readfiles']['Pulse_Repetition_Frequency (computed, Hz)']
                    first_line = int(self.datastack[date][swath][burst].processes['crop']['First_line (w.r.t. original_image)'])
                    last_line = int(self.datastack[date][swath][burst].processes['crop']['Last_line (w.r.t. original_image)'])

                    range_first = self.datastack[date][swath][burst].processes['readfiles']['Range_time_to_first_pixel (2way) (ms)']
                    range_samp = self.datastack[date][swath][burst].processes['readfiles']['Range_sampling_rate (computed, MHz)']
                    first_pixel = int(self.datastack[date][swath][burst].processes['crop']['First_pixel (w.r.t. original_image)'])
                    last_pixel = int(self.datastack[date][swath][burst].processes['crop']['Last_pixel (w.r.t. original_image)'])

                    no_lines = int(self.datastack[date][swath][burst].processes['readfiles']['Number_of_lines_original'])
                    no_pixels = int(self.datastack[date][swath][burst].processes['readfiles']['Number_of_pixels_original'])

                    # Calculate difference w.r.t. reference point.
                    range_time_diff = (float(range_first) - float(self.coordinates[date]['ref_range_time']))
                    pixel_diff = int(round((float(range_samp) * 1e3) * range_time_diff))

                    az_time1 = datetime.strptime(az_first, '%Y-%b-%d %H:%M:%S.%f')
                    az_time2 = datetime.strptime(self.coordinates[date]['ref_az_time'], '%Y-%b-%d %H:%M:%S.%f')
                    az_time_diff = (az_time1 - az_time2).total_seconds()
                    line_diff = int(round(float(az_samp) * az_time_diff))

                    # Calculate final corner coordinates.
                    ll = np.array([line_diff + first_line, pixel_diff + first_pixel], ndmin=2)
                    ul = np.array([line_diff + last_line, pixel_diff + first_pixel], ndmin=2)
                    ur = np.array([line_diff + last_line, pixel_diff + last_pixel], ndmin=2)
                    lr = np.array([line_diff + first_line, pixel_diff + last_pixel], ndmin=2)

                    self.coordinates[date][swath]['corners'][b,:,:] = np.vstack([ll,ul,ur,lr])
                    b += 1

                    # Check for max/min line/pixel to prevent negative pixel numbers.
                    min_line = min(1 + line_diff, min_line)
                    max_line = max(line_diff + no_lines, max_line)
                    min_pixel = min(1 + pixel_diff, min_pixel)
                    max_pixel = max(pixel_diff + no_pixels, max_pixel)

            if min_line < 1:
                max_line = max_line + (1 - min_line)
            if min_pixel < 1:
                max_pixel = max_pixel + (1 - min_pixel)

            for swath in self.datastack[date].keys():
                # If one of the lines or pixels is lower then 1, correct all coordinates to have positive coordinates for all
                # bursts.
                if min_line < 1:
                    self.coordinates[date][swath]['corners'][:,:,0] = self.coordinates[date][swath]['corners'][:,:,0] + (1 - min_line)
                if min_pixel < 1:
                    self.coordinates[date][swath]['corners'][:,:,1] = self.coordinates[date][swath]['corners'][:,:,1] + (1 - min_pixel)

                for burst in range(len(self.datastack[date][swath].keys())):
                    shape_c = self.coordinates[date][swath]['corners'][burst,:,:]
                    shape = box(shape_c[0,1],shape_c[0,0],shape_c[2,1],shape_c[2,0])
                    self.coordinates[date]['shapes'].append(shape)

                    # Finally add information to the .res file if already loaded
                    if self.datastack[date][swath]['burst_' + str(burst+1)].processes['readfiles']:

                        read = self.datastack[date][swath]['burst_' + str(burst+1)].processes['readfiles']
                        read['First_line (w.r.t. output_image)'] = str(shape_c[0,0])
                        read['Last_line (w.r.t. output_image)'] = str(shape_c[2,0])
                        read['First_pixel (w.r.t. output_image)'] = str(shape_c[0,1])
                        read['Last_pixel (w.r.t. output_image)'] = str(shape_c[2,1])
                        read['Number_of_pixels_output_image'] = str(max_pixel)
                        read['Number_of_lines_output_image'] = str(max_line)
                        self.datastack[date][swath]['burst_' + str(burst+1)].processes['readfiles'] = read
                    else:
                        print 'No resfile available, so information is not added to resfile'

    def write_stack(self,write_path='',no_data=False):
        # This function writes the full datastack to a given folder using the dates / swaths / bursts setup. This
        # also generates the res readfiles data.
        if write_path and os.path.exists(write_path):
            self.path = write_path
        if (not write_path or not os.path.exists(write_path)) and not self.path:
            warnings.warn('Please specify a path that exists to write the data')
            return

        write_jobs = []
        burst_num = []

        for date in self.datastack.keys():

            date_basic = date.translate(None,'-')
            date_path = os.path.join(self.path, date_basic)
            if not os.path.exists(date_path):
                os.mkdir(date_path)

            for swath in self.datastack[date].keys():

                swath_path = os.path.join(date_path, swath)
                if not os.path.exists(swath_path):
                    os.mkdir(swath_path)

                for burst in self.datastack[date][swath].keys():
                    burst_path = os.path.join(date_path, swath, burst)
                    if not os.path.exists(burst_path):
                        os.mkdir(burst_path)

                    # Finally write the bursts with their res files and precise orbits
                    xml = self.datastack[date][swath][burst].swath_xml
                    data = self.datastack[date][swath][burst].swath_data
                    image_no = str(self.datastack[date][swath][burst].burst_num)
                    stack_no = burst[6:]
                    xml_base = os.path.basename(xml)
                    res_name = os.path.join(burst_path, 'slave.res')
                    outdata = os.path.join(burst_path, 'slave_iw_' + xml_base[6] + '_burst_' + stack_no + '.raw')

                    self.datastack[date][swath][burst].write(res_name)
                    if not os.path.exists(res_name) or not os.path.exists(outdata):

                        write_jobs.append('python ' + self.function_path + 'sentinel_dump_data_function.py ' + data + ' ' + res_name + ' ' + outdata)
                        burst_num.append(stack_no + '_' + xml_base[6] + '_' + xml_base[15:23])

        # Burst are sorted in such a way that mainly read from different data files sorted by burst then swath then date.
        ids = sorted(range(len(burst_num)), key=lambda x: burst_num[x])

        jobList1 = []
        for id_val in ids:
            jobList1.append({"path": self.path, "command": write_jobs[id_val]})
            if not self.parallel:
                os.chdir(self.path)
                # Resample
                os.system(write_jobs[id_val])
        if self.parallel:
            jobs = Jobs(self.nr_of_jobs, self.doris_parameters)
            jobs.run(jobList1)

    def unpack_image(self, dest_folder=''):

        if not dest_folder:
            dest_folder = os.path.join(self.path, 'slc_data_files')
            self.unzip_path = dest_folder
        if not os.path.exists(dest_folder):
            os.mkdir(dest_folder)

        jobList1 = []

        # This program unpacks the images which are needed for processing. If unpacking fails, they are removed..
        for imagefile in self.images:

            zipped_folder = imagefile.zip_path
            if zipped_folder.endswith('.SAFE.zip'):
                imagefile.unzip_path = os.path.join(dest_folder, os.path.basename(zipped_folder[:-9] + '.SAFE'))
            elif zipped_folder.endswith('.zip'):
                imagefile.unzip_path = os.path.join(dest_folder, os.path.basename(zipped_folder[:-4] + '.SAFE'))
            shapefile = self.shape_filename
            pol = self.polarisation[0]
            overwrite = False
            command1 = ('python ' + self.function_path + 'load_shape_unzip.py ' + zipped_folder + ' ' + dest_folder +
                        ' ' + shapefile + ' ' + pol + ' ' + str(overwrite))
            jobList1.append({"path": self.path, "command": command1})
            if not self.parallel:
                os.chdir(self.path)
                # Resample
                os.system(command1)

        if self.parallel:
            jobs = Jobs(self.nr_of_jobs, self.doris_parameters)
            jobs.run(jobList1)

    def del_unpacked_image(self):
        # This program unpacks the images which are needed for processing.
        for image_dat in self.images:
            shutil.rmtree(image_dat.unzip_path)

    def swath_coverage(self):
        # Create a convex hull for the different swaths.

        for swath in self.swath_names:
            # Assign coverage of swath using convex hull.
            burst_id = [i for i in range(len(self.burst_names)) if swath in self.burst_names[i]]
            swath_shape = cascaded_union([self.burst_shapes[i] for i in burst_id])
            self.swath_shapes.append(swath_shape)

    def write_shapes(self,coverage=True,images=True,swaths=True,bursts=True):
        # This function writes shapefiles of the area of interest, images and bursts.

        if not coverage and not images and not bursts:
            warnings.warn('Select at least one of shape types')
            return

        shapes = list(); shape_names = list(); shape_files = list()
        if coverage and self.shape:
            shape_files.append(self.path + 'area_of_interest.shp')
            if self.shape.type == 'MultiPolygon':
                shapes.append([sh for sh in self.shape])
                shape_names.append([('coverage_with_buffer_of_' + str(self.buffer) + '_degrees_' + str(i)) for i in range(len(shapes[0]))])
            elif self.shape.type == 'Polygon':
                shape_names.append(['coverage_with_buffer_of_' + str(self.buffer) + '_degrees'])
                shapes.append([self.shape])
        if images and self.image_shapes:
            shape_files.append(self.path + 'image_coverage.shp')
            shape_names.append([date.astype(datetime).strftime('%Y-%m-%dT%H:%M:%S') for date in self.image_dates])
            shapes.append(self.image_shapes)
        if swaths and self.swath_shapes:
            shape_files.append(self.path + 'swath_coverage.shp')
            shape_names.append(self.swath_names)
            shapes.append(self.swath_shapes)
        if bursts and self.burst_shapes:
            shape_files.append(self.path + 'burst_coverage.shp')
            shape_names.append(self.burst_names)
            shapes.append(self.burst_shapes)

        shape_setup = {
            'geometry': 'Polygon',
            'properties': {'name': 'str'},
        }

        for i in range(len(shape_files)):
            with fiona.open(shape_files[i], 'w', 'ESRI Shapefile', shape_setup) as sh:
                for n in range(len(shapes[i])):
                    sh.write({
                        'geometry': mapping(shapes[i][n]),
                        'properties': {'name': shape_names[i][n]},
                    })



import argparse
import os
import xml.etree.ElementTree as ET
from doris.doris_stack.main_code.doris_sentinel_1 import DorisSentinel1

"""Doris processing
argument:  --parameterfilepath, -p
"""

# parse arguments here
parser = argparse.ArgumentParser(description='Doris processing.')
parser.add_argument('--parameterfilepath', '-p', default='./',
                    help='Path to dorisParameter.py file, this file contains case specific parameters')

args = parser.parse_args()

xml_file = os.path.join(os.path.join(args.parameterfilepath, 'doris_input.xml'))
print('Reading ' + xml_file)
tree = ET.parse(xml_file)
settings = tree.getroot()[0]

start_date = settings.find('.start_date').text
end_date = settings.find('.end_date').text
master_date = settings.find('.master_date').text

#start doris sentinel1 run
doris_sentinel_1 = DorisSentinel1()
doris_sentinel_1.run(args.parameterfilepath, start_date, end_date, master_date)



"""
This script is meant to cleanup datastacks and remove unnecessary intermediate results. The most general options are the
settings for a PS approach and a distributed scatterers approach, which is also used for atmospheric observations.
"""

import os
import shutil

# Test:
stack_folder = '/media/gert/Data/datastacks/netherlands/asc_t88/stack'
cleanup_ps = False
cleanup_ds = True
full_swath_rm = []
full_swath_keep = []
burst_rm = []
burst_keep = []
remove = False
files = cleanup(stack_folder, cleanup_ps, cleanup_ds, full_swath_rm, full_swath_keep, burst_rm, burst_keep, remove)

def cleanup(stack_folder, cleanup_ps=True, cleanup_ds=False, full_swath_rm=[], full_swath_keep=[], burst_rm=[],
            burst_keep=[], remove=True):
    """
    This is the main script to decide what should be removed and what not for an individual full swath.
    Basically a list is made of the files that should be removed from the full swath and burst folders based on the
    ending of the file name.
    The function returns a dictionary where are deleted files are listed. If you only want to list all the files that
    will be deleted use the remove=False option.

    Default options are:

    remove                          abbreviation            PS          DS
    interferogram                   ifg                     yes         yes
    ifg earth phase corrected       ifg_srp                 yes         yes
    ifg dem phase corrected         ifg_srd                 no          yes
    ifg phase filtered              ifg_filt                NA (yes)    no
    ifg coherence                   ifg_coh                 yes         no
    ifg unwrapped                   ifg_unw                 NA (yes)    no
    dem pixel shift (pixel)         dac_delta_p             yes         yes
    dem pixel shift (line)          dac_delta_l             yes         yes
    slave image                     s_ramp                  yes         yes
    slave image deramped            s_deramp                no          yes
    master image                    m_ramp                  yes         yes
    master image deramped           m_deramp                no (*)      yes
    dem phase                       dem                     no          no
    latitude                        phi                     no (*)      no
    longitude                       lam                     no (*)      no

    burst folder                    b_folder                yes         yes
        burst raw files             b_raw                   yes         yes
        burst ras files             b_ras                   yes         yes
        burst res files             b_res                   yes         yes

    * Only one needed in a single master stack. Is not implemented yet.
    """

    # First check what should be removed.
    if cleanup_ps:
        swath_clean = {'ifg': True, 'ifg_srp': True, 'ifg_srd': False, 'ifg_filt': True, 'ifg_coh': True,
                       'ifg_unw': True, 's_ramp': True, 's_deramp': False, 'm_ramp': True, 'm_deramp': False,
                       'dem': False, 'phi': False, 'lam': False, 'dac_delta_p': True, 'dac_delta_l': True}
        burst_clean = {'b_folder': True, 'b_raw': True, 'b_ras': True, 'b_res': True}
    elif cleanup_ds:
        swath_clean = {'ifg': True, 'ifg_srp': True, 'ifg_srd': True, 'ifg_filt': False, 'ifg_coh': False,
                       'ifg_unw': False, 's_ramp': True, 's_deramp': True, 'm_ramp': True, 'm_deramp': True,
                       'dem': False, 'phi': False, 'lam': False, 'dac_delta_p': True, 'dac_delta_l': True}
        burst_clean = {'b_folder': True, 'b_raw': True, 'b_ras': True, 'b_res': True}
    else:  # Otherwise nothing is removed unless indicated
        swath_clean = {'ifg': False, 'ifg_srp': False, 'ifg_srd': False, 'ifg_filt': False, 'ifg_coh': False,
                       'ifg_unw': False, 's_ramp': False, 's_deramp': False, 'm_ramp': False, 'm_deramp': False,
                       'dem': False, 'phi': False, 'lam': False, 'dac_delta_p': False, 'dac_delta_l': False}
        burst_clean = {'b_folder': False, 'b_raw': False, 'b_ras': False, 'b_res': False}

    # Add additional options.
    for dat in full_swath_rm:
        swath_clean[dat] = True
    for dat in full_swath_keep:
        swath_clean[dat] = False
    for dat in burst_rm:
        burst_clean[dat] = True
    for dat in burst_keep:
        burst_clean[dat] = False

    # Then create the strings with which these parts end
    swath_endings = {'ifg': 'int.raw', 'ifg_srp': 'srp.raw', 'ifg_srd': 'srd.raw', 'ifg_filt': 'filt.raw',
                     'ifg_coh': 'rence.raw', 'ifg_unw': 'unwrapped.raw', 's_ramp': 'rsmp.raw',
                     's_deramp': 'rsmp_deramped.raw', 'm_ramp': 'ster.raw', 'm_deramp': 'ster_deramped.raw',
                     'dem': 'dem_radar.raw', 'phi': 'phi.raw', 'lam': 'lam.raw',
                     'dac_delta_p': 'delta_pixel.raw', 'dac_delta_l': 'delta_line.raw'}
    burst_endings = {'b_folder': '', 'b_raw': '.raw', 'b_ras': '.ras', 'b_res': '.res'}

    # Finally, make a list of which endings should be deleted
    swath_remove = [dat for key, dat in swath_endings.iteritems() if swath_clean[key]]
    burst_remove = [dat for key, dat in burst_endings.iteritems() if burst_clean[key]]

    # Check the total ifgs in the stack
    swath_folders = scan_stack(stack_folder)

    # Go over these ifgs and remove intermediate steps in full swath and bursts.
    deleted = dict()
    for swath_folder in swath_folders:
        deleted[swath_folder] = dict()

        if burst_clean['b_folder']:
            folders = remove_burst_folders(swath_folder, remove)
            deleted[swath_folder]['folders'] = folders
        else:
            filenames = remove_burst_files(swath_folder, burst_remove, remove)
            deleted[swath_folder]['burst_files'] = filenames

        filenames = remove_file(swath_folder, swath_remove, remove)
        deleted[swath_folder]['swath_files'] = filenames

    return deleted


def scan_stack(stack_folder):
    # This function enters the children directories and checks whether a master.res, ifgs.res, swath folder and ifg
    # exist.

    swath_folders = []
    root, dirs, files = os.walk(stack_folder).next()

    for folder in dirs:
        r, folders, files = os.walk(os.path.join(root, folder)).next()

        if 'swath_1' in folders and 'master.res' in files and 'ifgs.res' in files and 'cint.raw' in files:
            swath_folders.append(os.path.join(root, folder))

    return swath_folders


def remove_burst_folders(swath_folder, remove):
    # Remove all burst folders from swath folder

    folder_names = []
    root, dirs, files = os.walk(swath_folder).next()

    for folder in dirs:
        if folder.startswith('swath'):
            if remove:
                shutil.rmtree(os.path.join(root, folder))
            folder_names.append(os.path.join(root, folder))

    return folder_names


def remove_file(swath_folder, file_endings, remove):
    # Remove the files in the main folder.

    file_names = []
    root, dirs, files = os.walk(swath_folder).next()

    for filename in files:
        for end in file_endings:
            if filename.endswith(end):
                if remove:
                    os.remove(os.path.join(root, filename))
                file_names.append(os.path.join(root, filename))

    return file_names


def remove_burst_files(swath_folder, file_endings, remove):
    # Remove the files in the burst folders.

    file_names = []
    root1, swaths, files = os.walk(swath_folder).next()

    if len(swaths) == 0:
        'Files seems to be deleted already!'
        return file_names

    for swath in swaths:
        root2, bursts, files = os.walk(os.path.join(root1, swath)).next()
        for burst in bursts:
            root3, burst_fold, files = os.walk(os.path.join(root2, burst)).next()
            for filename in files:
                for end in file_endings:
                    if filename.endswith(end) and remove:
                        if remove:
                            os.remove(os.path.join(root3, filename))
                        file_names.append(os.path.join(root3, filename))

    return file_names



import xml.etree.ElementTree as ET
import os
from datetime import datetime

class CreateDorisInputXml(object):

    def __init__(self, input_file):
        self.input_file_dict={}
        self.dem_folder=''
        if(len(input_file)==0):
            self._create_xml()
        else:
            self._read_xml(input_file)


    def _create_xml(self):
        # This will first create the framework with data folders the stackfolder should contain
        # a dorisparameters file.

        input = False
        while input == False:
            user_input = raw_input("Enter the path to the archive data folder: ")
            if os.path.exists(user_input):
                self.input_file_dict['sar_data_folder'] = user_input
                input = True
            else:
                print('The path is incorrect, use another path')

        input = False
        while input == False:
            user_input = raw_input("Which polarisation do you want to use (vv,hh,vh,hv): ")
            if user_input in ['vv', 'hh', 'vh', 'hv']:
                self.input_file_dict['polarisation'] = user_input
                input = True
            else:
                print('This polarisation does not exist')

        input = False
        while input == False:
            user_input = raw_input("Which track do you want to work with? (explore on https://scihub.copernicus.eu/dhus/) : ")
            try:
                input = str(int(user_input)).zfill(3)
                self.input_file_dict['track'] = user_input
                input = True
            except:
                print('This track does not exist')

        input = False
        while input == False:
            user_input = raw_input("Is this track ascending or descending? (asc/dsc) : ")
            if user_input in ['asc', 'dsc']:
                self.input_file_dict['direction'] = user_input
                input = True
            else:
                print('Input should either be asc or dsc')

        input = False
        while input == False:
            self.input_file_dict['datastack_folder'] = raw_input("Enter the path to the folder of new datastack: ")
            if os.path.exists(self.input_file_dict['datastack_folder']):
                input = True
            else:
                print('The path is incorrect, use another path')

        input = False
        while input == False:
            self.input_file_dict['shape_file_path'] = raw_input("Enter full path to the shapefile: ")
            if os.path.exists(self.input_file_dict['shape_file_path']) and self.input_file_dict['shape_file_path'].endswith('.shp'):
                input = True
            else:
                print('The path is incorrect, use another path')

        input = False
        while input == False:
            user_input = raw_input("Enter the path to the folder of the orbit files: ")
            if os.path.exists(user_input):
                self.input_file_dict['orbits_folder'] = user_input
                input = True
            else:
                print('The path is incorrect, use another path')

        input = False
        while input == False:
            user_input = raw_input("Do you want to generate the DEM file automaticly (Yes/No): ").lower()
            if user_input == 'yes' or user_input == 'no':
                self.input_file_dict['generate_dem'] = user_input
                input = True
            else:
                print('You should use either yes or no')

        input = False
        while input == False:
            self.input_file_dict['dem_processing_folder'] = raw_input("Enter path to the dem folder: ")
            self.input_file_dict['dem_folder'] = os.path.join(self.input_file_dict['datastack_folder'], 'dem')
            if os.path.exists(self.input_file_dict['dem_processing_folder']):
                input = True
            else:
                print('The path is incorrect, use another path')

        input = False
        while input == False:
            user_input = raw_input("Do you want to use parallel computing (Yes/No): ").lower()
            if user_input == 'yes' or user_input == 'no':
                self.input_file_dict['parallel'] = user_input
                input = True
            else:
                print('You should use either yes of no')

        if user_input == 'yes':
            nodes = raw_input("How many cores do you want to use: ")
            self.input_file_dict['cores'] = nodes

        input = False
        while input == False:
            user_input = raw_input("What is the start date of your stack in yyyy-mm-dd (can be changed later): ").lower()
            try:
                date = datetime.strptime(user_input, '%Y-%m-%d')
                self.input_file_dict['start_date'] = user_input
                input = True
            except:
                print('Format not recognized, 01-01-2014 chosen')
                self.input_file_dict['start_date'] = user_input

        input = False
        while input == False:
            user_input = raw_input("What is the end date of your stack in yyyy-mm-dd (can be changed later): ").lower()
            try:
                date = datetime.strptime(user_input, '%Y-%m-%d')
                self.input_file_dict['end_date'] = user_input
                input = True
            except:
                print('Format not recognized, 01-01-2050 chosen')
                self.input_file_dict['end_date'] = user_input

        input = False
        while input == False:
            user_input = raw_input("What is the master date of your stack in yyyy-mm-dd (can be changed later): ").lower()
            try:
                date = datetime.strptime(user_input, '%Y-%m-%d')
                self.input_file_dict['master_date'] = user_input
                input = True
            except:
                print('Format not recognized, 01-01-2016 chosen. Check https://scihub.copernicus.eu/dhus/#/home for valid date')
                self.input_file_dict['master_date'] = user_input

        xml_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'doris_input_template.xml')
        tree = ET.parse(xml_file)
        self.settings = tree.getroot()

        for key in self.input_file_dict.keys():
            self.settings.find('*/' + key).text = self.input_file_dict.get(key)

        tree.write(os.path.join(self.input_file_dict['datastack_folder'], 'doris_input.xml'))

        return self.input_file_dict

    def _read_xml(self, input_file):
        xml_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), input_file)
        tree = ET.parse(xml_file)
        self.settings = tree.getroot()

    def get_value(self, key):
        return self.settings.find('*/' + key).text



import numpy as np
from numpy import *
from doris.doris_stack.functions.get_ramp import get_ramp, get_parameter
import os


########################################################################################################################
# Function to get parameters from files
# Parameter = get_parameter(First_param,file_name,format_flag=1,Second_param=None,Third_param=None)
def get_parameter(First_param,file_name,format_flag=1,Second_param=None,Third_param=None):
    Read_contine_flag=0
    class set_class(object):
        pass
    orbit_info = set_class()
    time_temp = []
    x_temp = []
    y_temp = []
    z_temp = []
    value=None

    for line in open(file_name):
        if format_flag==1:
            if not (line.find(First_param)):
                index=line.find(':')
                value=(line[(index+1):].strip(' \n\t'))
                return value

        if format_flag==2:
            if  not (line.find(Second_param)):
                Read_contine_flag=1
            if (Read_contine_flag==1) and (not (line.find(First_param))):  ##Be careful
                index=line.find(':')
                value=(line[(index+1):].strip(' \n\t'))
            if  (not (line.find(Third_param))):  ##Be careful
                Read_contine_flag=0
                return value

        if format_flag==3:
            if not (line.find(First_param)):
                index=line.find(':')
                pixel_time=(line[(index+1):].strip(' \n\t')).split(' ')[1].split(':')
                return pixel_time

        if format_flag==4:
            if not (line.find(First_param)):
                index=line.find(':')
                value=int(line[(index+1):].strip(' \n\t'))
                Read_contine_flag=1
                continue
            if (Read_contine_flag>=1) :
                new_line = line.strip('\n').split()
                time_temp.append(float(new_line[0]))
                x_temp.append(float(new_line[1]))
                y_temp.append(float(new_line[2]))
                z_temp.append(float(new_line[3]))
                Read_contine_flag=Read_contine_flag+1
                if (Read_contine_flag==(value+1)):
                    setattr(orbit_info,'x',x_temp)
                    setattr(orbit_info,'y',y_temp)
                    setattr(orbit_info,'z',z_temp)
                    setattr(orbit_info,'time',time_temp)
                    return orbit_info

########################################################################################################################
# Definition to extract data
# thisBurstData = freadbk(path_file,line_start=1, pixel_start=1, nofLines=None, nofPixels=None, dt=np.dtype(np.float32), lines=0, pixels=0):
def freadbk(path_file, line_start=1, pixel_start=1, nofLines=None, nofPixels=None, dt='float32', lines=0, pixels=0, memmap=True):
    # First use memmap to get a memory map of the full file, than extract the requested part.

    if dt == 'cpxint16':
        dtype = np.dtype([('re', np.int16), ('im', np.int16)])
        file_dat = np.memmap(path_file, dtype=dtype, mode='r', shape=(lines, pixels)).view(np.int16).astype(np.float32).view(np.complex64)
        data = file_dat[line_start - 1:line_start + nofLines - 1, pixel_start - 1:pixel_start + nofPixels - 1].astype(
            'complex64', subok=False)
    elif dt == 'cpxshort':

        file_dat = np.memmap(path_file, dtype=np.dtype(np.float16), mode='r', shape=(lines, pixels * 2))
        data = 1j * file_dat[:, 1::2].astype('float32', subok=False)
        data += file_dat[:, 0::2].astype('float32', subok=False)
        data = data[line_start - 1:line_start + nofLines - 1, pixel_start - 1:pixel_start + nofPixels - 1]

    else:
        dt = np.dtype(dt)
        file_dat = np.memmap(path_file, dtype=dt, mode='r', shape=(lines, pixels))
        data = file_dat[line_start - 1:line_start + nofLines - 1, pixel_start - 1:pixel_start + nofPixels - 1].astype(
            dt, subok=False)

    return data


########################################################################################################################
# Function to calculate normalized Doppler Centroid frequency
# Df_DC = get_f_DC_difference(f_DC_1, f_DC_2, BOL_Length, BOL_lines, PRF, normalize)
def get_f_DC_difference(nBurst):

    burst1 = 'burst_' + str(nBurst) + '/'
    burst2 = 'burst_' + str(nBurst + 1) + '/'

    this_m_resData = burst1 + 'master.res'
    next_m_resData = burst2 + 'master.res'

    os.chdir(os.getcwd() + '/' + burst1)
    f_DC_1 = get_ramp(os.path.basename(this_m_resData), resampled=0, type='DC')
    os.chdir(os.path.dirname(os.getcwd()))

    os.chdir(os.getcwd() + '/' + burst2)
    f_DC_2 = get_ramp(os.path.basename(next_m_resData), resampled=0, type='DC')
    os.chdir(os.path.dirname(os.getcwd()))

    line_start, line_length, first_pixel_this, first_pixel_next, pixel_length, this_nr_oflines, this_nr_ofpixels, next_nr_oflines, next_nr_ofpixels, PRF = get_coordinates(nBurst)

    Df_DC = f_DC_1[line_start - 1:line_start + line_length - 1, first_pixel_this - 1:first_pixel_this + pixel_length - 1] - \
            f_DC_2[0:line_length, first_pixel_next - 1: first_pixel_next + pixel_length - 1]

    return Df_DC

def get_coordinates(nBurst):

    burst1 = 'burst_' + str(nBurst) + '/'
    burst2 = 'burst_' + str(nBurst+1) + '/'
    this_m_resData = burst1 + 'master.res'
    next_m_resData = burst2 + 'master.res'

    # Get variables from first burst
    this_line_first     = int(get_parameter('First_line (w.r.t. output_image)',this_m_resData,1))
    this_line_last      = int(get_parameter('Last_line (w.r.t. output_image)',this_m_resData,1))
    this_nr_oflines     = int(this_line_last) - int(this_line_first) +1
    this_pixel_first    = int(get_parameter('First_pixel (w.r.t. output_image)',this_m_resData,1))
    this_pixel_last     = int(get_parameter('Last_pixel (w.r.t. output_image)',this_m_resData,1))
    this_nr_ofpixels    = int(this_pixel_last) - int(this_pixel_first) +1
    PRF_1               = float(get_parameter('Pulse_Repetition_Frequency (computed, Hz)',this_m_resData,1))

    # Get variables from second burst
    next_line_first     = int(get_parameter('First_line (w.r.t. output_image)',next_m_resData,1))
    next_line_last      = int(get_parameter('Last_line (w.r.t. output_image)',next_m_resData,1))
    next_nr_oflines     = int(next_line_last) - int(next_line_first) +1
    next_pixel_first    = int(get_parameter('First_pixel (w.r.t. output_image)',next_m_resData,1))
    next_pixel_last     = int(get_parameter('Last_pixel (w.r.t. output_image)',next_m_resData,1))
    next_nr_ofpixels    = int(next_pixel_last) - int(next_pixel_first) +1

    PRF = PRF_1

    # Read only the Burstoverlap
    if this_pixel_first < next_pixel_first:
        first_pixel = next_pixel_first
    elif this_pixel_first >= next_pixel_first:
        first_pixel = this_pixel_first
    if this_pixel_last > next_pixel_last:
        pixel_length = next_pixel_last - first_pixel + 1
    elif this_pixel_last <= next_pixel_last:
        pixel_length = this_pixel_last - first_pixel + 1

    first_pixel_this = first_pixel - this_pixel_first + 1
    first_pixel_next = first_pixel - next_pixel_first + 1

    line_length = this_line_last - next_line_first + 1
    line_start = this_nr_oflines - line_length + 1

    return line_start, line_length, first_pixel_this, first_pixel_next, pixel_length, this_nr_oflines, this_nr_ofpixels,\
           next_nr_oflines, next_nr_ofpixels, PRF


########################################################################################################################
# Function to calculate the interferograms of both bursts
# thisburstdata, nextburstdata, diffBursts, PRF = get_interfero(nBurst, BOL_lines, BOL_Length)
def get_offset(nBurst, Df_DC, coh_treshold=0.3):

    burst1 = 'burst_' + str(nBurst) + '/'
    burst2 = 'burst_' + str(nBurst+1) + '/'

    # cpxint16 and cpxfloat32
    dataFormat_s = 'complex64'

    line_start, line_length, first_pixel_this, first_pixel_next, pixel_length, this_nr_oflines, this_nr_ofpixels, next_nr_oflines, next_nr_ofpixels, PRF = get_coordinates(nBurst)

    ifgs_1  = freadbk(burst1 + 'cint.raw.old', line_start, first_pixel_this, line_length, pixel_length , dataFormat_s,  this_nr_oflines, this_nr_ofpixels)
    ESD_coh_1  = freadbk(burst1 + 'coherence.raw', line_start, first_pixel_this, line_length, pixel_length , 'float32',  this_nr_oflines, this_nr_ofpixels)
    ifgs_2  = freadbk(burst2 + 'cint.raw.old', 1, first_pixel_next, line_length, pixel_length, dataFormat_s, next_nr_oflines, next_nr_ofpixels)
    ESD_coh_2 = freadbk(burst2 + 'coherence.raw', 1, first_pixel_next, line_length,
                     pixel_length, 'float32', next_nr_oflines, next_nr_ofpixels)
    ESD_coh = (ESD_coh_1 + ESD_coh_2) / 2

    #ifgs_1_total = freadbk(burst1 + 'cint.raw.old', 1, 1, this_nr_oflines, this_nr_ofpixels, dataFormat_s,  this_nr_oflines, this_nr_ofpixels)
    #ifgs_2_total = freadbk(burst2 + 'cint.raw.old', 1, 1, next_nr_oflines, next_nr_ofpixels, dataFormat_s,  next_nr_oflines, next_nr_ofpixels)

    # Remove invalid data both in range and azimuth
    valid_range = []
    valid_azimuth = []
    for i in range(0,len(ifgs_1[0,:])):
        if np.nanmean(abs(ifgs_1[:,i])) != 0 and np.nanmean(abs(ifgs_2[:,i])) != 0:
            valid_range.append(i)

    for i in range(0,len(ifgs_1[:,0])):
        if np.nanmean(abs(ifgs_1[i,:])) != 0 and np.nanmean(abs(ifgs_2[i,:])) != 0:
            valid_azimuth.append(i)

    if valid_range and valid_azimuth:
        ifgs_1 = ifgs_1[:, valid_range[:]]
        ifgs_2 = ifgs_2[:, valid_range[:]]
        ESD_coh = ESD_coh[:, valid_range[:]]

        ifgs_1 = ifgs_1[valid_azimuth[:], :]
        ifgs_2 = ifgs_2[valid_azimuth[:], :]
        ESD_coh = ESD_coh[valid_azimuth[:], :]

        Df_DC = Df_DC[:, valid_range[:]]
        Df_DC = Df_DC[valid_azimuth[:], :]

    # First downsample 2 * 10
    Nra = 10
    Naz = 2
    new_ra = ESD_coh.shape[1] / 10
    new_az = ESD_coh.shape[0] / 2

    ESD_coh = ESD_coh[0:new_az*Naz-1:Naz, 0:new_ra*Nra-1:Nra]
    ifgs_1_multilook = ifgs_1[:new_az*2, :new_ra*10].reshape([new_az, Naz, new_ra, Nra]).mean(3).mean(1)
    ifgs_2_multilook = ifgs_2[:new_az*2, :new_ra*10].reshape([new_az, Naz, new_ra, Nra]).mean(3).mean(1)
    Df_DC_multilook = Df_DC[:new_az * 2, :new_ra * 10].reshape([new_az, Naz, new_ra, Nra]).mean(3).mean(1)

    # Double difference and calculate weights according to Cramer Rao bound
    diffBursts = ifgs_1_multilook * ifgs_2_multilook.conj()
    weights = 2 * ESD_coh*ESD_coh / (1 - ESD_coh*ESD_coh)

    W = np.sum(weights[ESD_coh > coh_treshold])
    angle = (PRF / (2 * np.pi * np.nanmean(Df_DC_multilook[ESD_coh > coh_treshold] * weights[ESD_coh > coh_treshold] /
                                            np.mean(weights[ESD_coh > coh_treshold]))))
    offset = np.angle(np.sum(diffBursts[ESD_coh > coh_treshold] * weights[ESD_coh > coh_treshold]) / W) * angle

    angle_pixel = angle * (line_start - 1)

    return offset, W, angle_pixel

########################################################################################################################
# Function to calculate pixel offset for each burst, according to Nida's proposed method. Threshold can be assigned by
# user.
# offset = apply_ESD_Nida(diffBursts, Df_DC, PRF, threshold)
def apply_ESD_Nida(diffBursts, Df_DC, PRF, threshold = 0.0001):

    # Determine phase of interferogram
    ph_esd = np.angle(diffBursts)

    # Do an intitial estimation based on the peak of the histogram
    N,X = np.histogram(ph_esd[:], 50)
    idx = np.argmax(N)
    D_az_init = X[idx]*(PRF/(2*np.pi*np.nanmean(Df_DC[:])))

    # Create linspace from -0.1 to 0.1 in 7 steps
    D_az_span = -0.3*np.pi*(PRF/(2*np.pi*np.nanmean(Df_DC[:])))
    D_azs = np.linspace(D_az_init-D_az_span, D_az_init+D_az_span,num=7)
    del D_az_span

    c = -1
    D_az_min = []
    # Keeps refinining until error with earlier estimation is lower than 0.0001
    while True:
        c += 1
        ph_test = np.ones(len(D_azs))
        # Calculate estimated phase, residual phase and test phase
        for k in range(0,len(D_azs)):
            D_az = D_azs[k]
            ph_est = (2*np.pi*Df_DC*D_az)/PRF
            ph_res = ph_esd - ph_est

            ph_test[k] = np.nanmean(np.angle(exp(1j * ph_res[:]))) # should be ph_test(k) = np.nanmean(exp(1i*ph_res[:]))
            #print ph_test

        ind = np.argmin(abs(ph_test))
        D_az_min.append(D_azs[ind])

        # Break iteration when pixel shift is sufficient
        if c > 2 and abs(D_az_min[c]-D_az_min[c-1]) < threshold and abs(D_az_min[c-1]-D_az_min[c-2]) < threshold:
            ph_est_opt = np.nanmean(ph_est[:])
            offset = -D_az_min[c]
            break

        # Use a smaller difference for next iteration
        D_az_span = D_azs[1] - D_azs[0]
        D_azs = np.linspace(D_azs[ind]-D_az_span, D_azs[ind]+D_az_span,num=7)
        del ph_test

    #print 'amount of loops in iteration ' + str(c)

    pix_offset = offset / (PRF/(2*np.pi*np.nanmean(Df_DC[:])))

    return offset, pix_offset



from datetime import datetime
import os
from doris.doris_stack.main_code.grs_config import GrsConfig
import xml.etree.ElementTree as ET


class DorisParameters():

    """This class contains all parameters that are used in the execution of test_dat_EDS_5.py.
       path parameters are checked for existance,
       all parameters are printed to stdout
    """

    def __init__(self, stack_path):

        grs_config = GrsConfig()

        self.doris_path = grs_config.doris_path
        self.cpxfiddle_path = grs_config.cpxfiddle_path
        self.job_handler_script = grs_config.job_handler_script
        self.function_path = grs_config.function_path
        self.source_path = grs_config.source_path

        self.verbose = True

        tree = ET.parse(os.path.join(stack_path, 'doris_input.xml'))
        self.settings = tree.getroot()

        archive_path = self._settings_get('.sar_data_folder')
        self.archive_path = archive_path
        project_path = self._settings_get('.datastack_folder')
        self.project_path = project_path
        data_path = self._settings_get('.sar_data_folder')
        self.data_path = data_path
        polarisation = self._settings_get('.polarisation')
        self.polarisation = polarisation
        track = self._settings_get('.track')
        self.track = track
        direction = self._settings_get('.direction')
        self.direction = direction
        #
        # used in single_master.py
        #
        #
        # used in test_dat_ESD
        #
        # TODO DLE FIX shape path
        self.shape_dat = self._settings_get('.shape_file_path')
        self.track_dir = data_path
        self.stack_path = os.path.join(project_path, 'stack')
        self.precise_orbits = self._settings_get('.orbits_folder')
        # Start data of datastack. If end date not given it searches till current.
        self.input_files = os.path.join(project_path, 'input_files')

        self.parallel = self._settings_compare('.parallel', 'yes')
        self.nr_of_jobs = int(self._settings_get('.cores'))

        self.profile_log = project_path + '/profile_log'
        self.doris_parallel_flag_dir = project_path + '/.Doris_parallel'
        self.between_sleep_time = 1
        self.end_sleep_time = 1

        self.do_coarse_orbits = self._settings_compare('.do_coarse_orbits', 'yes')
        self.do_deramp = self._settings_compare('.do_deramp', 'yes')
        self.do_fake_fine_coreg_bursts = self._settings_compare('.do_fake_fine_coreg_bursts', 'yes')
        self.do_dac_bursts = self._settings_compare('.do_dac_bursts', 'yes')
        self.do_fake_coreg_bursts = self._settings_compare('.do_fake_coreg_bursts', 'yes')
        self.do_resample = self._settings_compare('.do_resample', 'yes')
        self.do_reramp = self._settings_compare('.do_reramp', 'yes')
        self.do_interferogram = self._settings_compare('.do_interferogram', 'yes')
        self.do_compref_phase = self._settings_compare('.do_compref_phase', 'yes')
        self.do_compref_dem = self._settings_compare('.do_compref_dem', 'yes')
        self.do_coherence = self._settings_compare('.do_coherence', 'yes')
        self.do_esd = self._settings_compare('.do_esd', 'yes')
        self.do_network_esd = self._settings_compare('.do_network_esd', 'yes')
        self.do_ESD_correct = self._settings_compare('.do_ESD_correct', 'yes')
        self.do_ref_phase = self._settings_compare('.do_ref_phase', 'yes')
        self.do_ref_dem = self._settings_compare('.do_ref_dem', 'yes')
        self.do_phasefilt = self._settings_compare('.do_phasefilt', 'yes')
        self.do_calc_coordinates = self._settings_compare('.do_calc_coordinates', 'yes')
        self.do_multilooking = self._settings_compare('.do_multilooking', 'yes')
        self.do_unwrap = self._settings_compare('.do_unwrap', 'yes')
        #
        # used in Jobs
        #
        # self.job_handler_script = source_path + "/sentinel1/main_code/jobHandlerScript"
        #
        # Print parameters, check if paths exist
        #

        print 'self.shape_dat: ' + self.shape_dat
        # self._check_path_exists(self.shape_dat)
        print 'self.track_dir:	' + self.track_dir
        self._check_path_exists(self.track_dir)
        print 'self.stack_path:	' + self.stack_path
        # self._check_path_exists(self.stack_path)
        print 'self.precise_orbits:	' + self.precise_orbits
        self._check_path_exists(self.precise_orbits)
        print 'self.input_files:	' + self.input_files
        # self._check_path_exists(self.input_files)
#        print 'self.main_code_folder:	' + self.main_code_folder
#        self._check_path_exists(self.main_code_folder)
#        print 'self.script_folder:	' + self.script_folder
#        self._check_path_exists(self.script_folder)
        print 'self.nr_of_jobs:	' + str(self.nr_of_jobs)
        print 'self.jobHandlerScript:	' + self.job_handler_script
        self._check_path_exists(self.job_handler_script)

    def _check_path_exists(self, path):
        if not(os.path.exists(path)):
            print 'Error Doris_Parameters: path ' + path + ' does not exist'
            
    def _settings_get(self, string):
        return self.settings.find('*/' + string).text


    def _settings_compare(self, string, comp_string):
        if (self.settings.find('*/' + string).text.lower()==comp_string.lower()):
            return True
        return False



#!/usr/bin/env python
import numpy as np
from numpy import *
import gdal
from gdalconst import *
from scipy import ndimage

def usage():
    print '\nUsage: python get_winpos.py dataFile resFile Nwin outFile                        '
    print 'where   dataFile           is the name of burst you want to deramp                 '
    print '        resFile            is the .res file of burst                               '
    print '        Nwin               number of windows to be distributed over the total image'
    print '        outFile            output file name                                        '
    print '  for example                                                                      '
    print ' python get_winpos.py 20141003_iw_1_burst_1.raw 20141003_iw_1_burst_1.res 2001 winpos_fine.asc'
    print ' matlab: TU Delft                                                                  '
    print ' Python: Wu Wenhao   Wuhan QQ:460249274                                            '
try:
    dataFile   = sys.argv[1]
    resFile    = sys.argv[2]
    Nwin       = sys.argv[3]
    outFile    = sys.argv[4]
  
except:
    print 'Unrecognized input'
    usage()
    sys.exit(1)


################################################################################
def get_parameter(First_param,file_name,format_flag=1,Second_param=None,Third_param=None):
    Read_contine_flag=0
    orbit_info=""
    value=None
    for line in open(file_name):
        if format_flag==1:
            if not (line.find(First_param)):
                index=line.find(':')
                value=(line[(index+1):].strip(' \n\t'))
                return value 

        if format_flag==2:
            if not(line.find(Second_param)):
                Read_contine_flag=1             
            if (Read_contine_flag==1) and (not (line.find(First_param))):  #Be careful
                index=line.find(':')
                value=(line[(index+1):].strip(' \n\t'))
                continue               
            if Read_contine_flag==1  and (not(line.find(Third_param))):  #Be careful               
                Read_contine_flag=0         
                return value
               

        if format_flag==3:            
            if not (line.find(First_param)):
                index=line.find(':')
                pixel_time=(line[(index+1):].strip(' \n\t')).split(' ')[1].split(':')
                return pixel_time  


        if format_flag==4:
            
            if not (line.find(First_param)):
                index=line.find(':')                
                value=int(line[(index+1):].strip(' \n\t'))
                Read_contine_flag=1
                continue                 
            if (Read_contine_flag>=1):               
                orbit_info=orbit_info+line
                Read_contine_flag=Read_contine_flag+1
                if (Read_contine_flag==(value+1)):
                    return orbit_info      
###############################################################################
#thisBurstData = freadbk(['burst' num2str(nBurst)   '/cint_srd.raw'],nofLines1,formatData1, line1, nofLines1,1,nofPixels1);
def freadbk(path_file,line_start=1, Pixels_start=1,nofLines1=None,nofPixels1=None):
    #Driver
    driver=gdal.GetDriverByName('MFF')
    driver.Register()
    gdal.AllRegister()
    thisBurstData_file=gdal.Open(path_file,GA_ReadOnly)
    if thisBurstData_file is None:
        print 'Could not open'+Path_MFF_HDR
        sys.exit(1)
    #print 'Driver: ', thisBurstData_file.GetDriver().ShortName,'/', \
    #      thisBurstData_file.GetDriver().LongName
    #print 'Size is ',thisBurstData_file.RasterXSize,'x',thisBurstData_file.RasterYSize, \
    #      'x',thisBurstData_file.RasterCount
    #print 'Projection is ',thisBurstData_file.GetProjection()
    geotransform = thisBurstData_file.GetGeoTransform()
    #if not geotransform is None:
    #    print 'Origin = (',geotransform[0], ',',geotransform[3],')'
    #    print 'Pixel Size = (',geotransform[1], ',',geotransform[5],')'

    cint_srd=thisBurstData_file.GetRasterBand(1)
    #print 'Band Type=',gdal.GetDataTypeName(cint_srd.DataType)

    if cint_srd.GetOverviewCount() > 0:
            print 'Band has ', cint_srd.GetOverviewCount(), ' overviews.'
    thisBurstData= cint_srd.ReadAsArray(int(Pixels_start-1),int(line_start-1),nofPixels1,nofLines1)
    return thisBurstData
###############################################################################


NwinGrid = 5

azSpacing = 20
rSpacing  = 5


resData   = resFile

if get_parameter('First_line (w.r.t. ovs_image)',resData,1): #%oversampled data
  #% First_line (w.r.t. ovs_image):
    l0 = int(get_parameter('First_line (w.r.t. ovs_image)',resData,1));
  #% Last_line (w.r.t. ovs_image):
    lN = int(get_parameter('Last_line (w.r.t. ovs_image)',resData,1));
  #% First_pixel (w.r.t. ovs_image):
    p0 = int(get_parameter('First_pixel (w.r.t. ovs_image)',resData,1));
  #% Last_pixel (w.r.t. ovs_image):
    pN = int(get_parameter('Last_pixel (w.r.t. ovs_image)',resData,1));
  
    dataFormat = 'cpxfloat32'
   
else:#original data
    l0 = int(get_parameter('First_line (w.r.t. original_image)',resData,1));
    #% Last_line (w.r.t. original_image):
    lN = int(get_parameter('Last_line (w.r.t. original_image)',resData,1))
    #% First_pixel (w.r.t. original_image):
    p0 = int(get_parameter('First_pixel (w.r.t. original_image)',resData,1))
    #% Last_pixel (w.r.t. original_image):
    pN = int(get_parameter('Last_pixel (w.r.t. original_image)',resData,1))
    dataFormat = 'cpxint16'


# Image size
Nlines = lN-l0+1;
Npixels = pN-p0+1;
print "Nlines =",Nlines 
print "Npixels =",Npixels


Ngrid = float(Nwin)/NwinGrid;
daz = Nlines*azSpacing;
dr = Npixels*rSpacing;

ratio = float(dr)/daz

Ngrid_az = sqrt(Ngrid/ratio);
Ngrid_r = round(Ngrid_az*ratio);
Ngrid_az = round(Ngrid_az)


Nlines_grid = ceil(Nlines/Ngrid_az)
Nlines_grid_orig = Nlines_grid


Npixels_grid = ceil(Npixels/Ngrid_r)
Npixels_grid_orig = Npixels_grid


RAW_CINT_SRD  = dataFile
Path_MFF_HDR  = dataFile.split('.')[0]+'.hdr';


if dataFormat == 'cpxint16':
   Link_CINT_SRD=dataFile.split('.')[0]+'.j00'
else:
   Link_CINT_SRD=dataFile.split('.')[0]+'.x00'


outStream      = open(Path_MFF_HDR,'w')
outStream.write('IMAGE_FILE_FORMAT = MFF\n')
outStream.write('FILE_TYPE = IMAGE\n')
outStream.write('IMAGE_LINES = %d\n' % int(Nlines))
outStream.write('LINE_SAMPLES = %d\n'% int(Npixels))
outStream.write('BYTE_ORDER = LSB\n')
outStream.write('END\n')
outStream.close()
    
if (os.path.exists(Link_CINT_SRD)):
    os.remove(Link_CINT_SRD)
RAW_CINT_SRD_ABSOLUTE_PATH=os.path.abspath(RAW_CINT_SRD)
print "RAW_CINT_SRD_ABSOLUTE_PATH=", RAW_CINT_SRD_ABSOLUTE_PATH
os.symlink(RAW_CINT_SRD_ABSOLUTE_PATH,Link_CINT_SRD)


winpos=np.array([],dtype='int32').reshape(0,2)


for v in range(1,int(Ngrid_az)+1):

    if v==Ngrid_az:
       Nlines_grid = (Nlines-1)%Nlines_grid_orig+1
    else:
       Nlines_grid = Nlines_grid_orig   
    ampArray = abs(freadbk(Path_MFF_HDR,int((v-1)*Nlines_grid_orig+1),1,int(Nlines_grid),Npixels ))
    for w in range(1,int(Ngrid_r)+1):
        if w==Ngrid_r:
            Npixels_grid = (Npixels-1)%Npixels_grid_orig+1
        else:
            Npixels_grid = Npixels_grid_orig
        amp = ampArray[:,(w-1)*Npixels_grid_orig:(w-1)*Npixels_grid_orig+Npixels_grid]
        locMaxsInd = amp == ndimage.grey_dilation(amp, size=(5*rSpacing, 5*azSpacing))       
        locMaxs = amp[locMaxsInd]
        [az,r] = where(locMaxsInd)
        sortIdx =np.argsort(-locMaxs)        
        sortIdx = sortIdx[0:NwinGrid]     
        add_winpos=np.array([az[sortIdx]+l0-1+(v-1)*Nlines_grid_orig,r[sortIdx]+p0-1+(w-1)*Npixels_grid_orig]).transpose()     
        winpos=np.vstack([winpos,add_winpos])

fidRes        = open(outFile,'w')
cols = winpos.shape[1]
rows = winpos.shape[0]        
#print "cols = ",cols
print "rows = ", rows
for i_temp in range(0,rows):       
    fidRes.write( '%d   %d\n' % (winpos[i_temp,0]+1,winpos[i_temp,1]+1))      
fidRes.close()

if (os.path.exists(Link_CINT_SRD)):
    os.remove(Link_CINT_SRD)
if (os.path.exists(Path_MFF_HDR)):
    os.remove(Path_MFF_HDR)
